// import formatDistance from './_lib/formatDistance/index'
// import formatLong from './_lib/formatLong/index'
// import formatRelative from './_lib/formatRelative/index'
// import localize from './_lib/localize/index'
// import match from './_lib/match/index'

/**
 * @type {Locale}
 * @category Locales
 * @summary Filipino locale.
 * @language Filipino
 * @iso-639-2 fil
 * @author Ian De La Cruz [@RIanDeLaCruz]{@link https://github.com/RIanDeLaCruz}
 */
// var locale = {
//   code: 'fil',
//   formatDistance: formatDistance,
//   formatLong: formatLong,
//   formatRelative: formatRelative,
//   localize: localize,
//   match: match,
//   options: {
//     weekStartsOn: 0 /* Sunday */,
//     firstWeekContainsDate: 1
//   }
// }

// export default locale

throw new Error(
  'fil locale is currently unavailable. Please check the progress of converting this locale to v2.0.0 in this issue on Github: TBA'
)
