/**
 * @author Gabor X. Toth
 * @author Marcello Stanisci
 * @author Florian Dold
 */

body {
  /* min-height: 20em; */
  /* width: 30em; */
  /* max-height: 800px; */
  margin: 0;
  font-size: 100%;
  padding: 0;
  overflow: hidden;
  background-color: #f8faf7;
  font-family: Arial, Helvetica, sans-serif;
}
/* taler-popup {
  min-height: 20em;
  width: 30em;
  height: 20rem;
  margin: 0;
  font-size: 100%;
  padding: 0;
  overflow: hidden;
  background-color: #f8faf7;
  font-family: Arial, Helvetica, sans-serif;
} */

.nav {
  background-color: #033;
  /* padding: 0.5em 0; */
}

.nav a {
  color: #f8faf7;
  padding-top: 0.7em;
  display: inline-block;
  width: calc(400px / 5);
  padding-bottom: 0.7em;
  text-align: center;
  text-decoration: none;
}

.nav a.active {
  background-color: #f8faf7;
  color: #000;
  font-weight: bold;
}

.container {
  overflow-y: scroll;
  max-height: 400px;
}

.abbrev {
  text-decoration-style: dotted;
}

#content {
  padding: 1em;
}

#wallet-table .amount {
  text-align: right;
}

.hidden {
  display: none;
}

#transactions-table th,
#transactions-table td {
  padding: 0.2em 0.5em;
}

#reserve-create table {
  width: 100%;
}

#reserve-create table td.label {
  width: 5em;
}

#reserve-create table .input input[type="text"] {
  width: 100%;
}

.historyItem {
  min-width: 380px;
  display: flex;
  flex-direction: row;
  border-bottom: 1px solid #d9dbd8;
  padding: 0.5em;
  align-items: center;
}

.historyItem .amount {
  font-size: 110%;
  font-weight: bold;
  text-align: right;
}

.historyDate,
.historyTitle,
.historyText,
.historySmall {
  margin: 0.3em;
}

.historyDate {
  font-size: 90%;
  color: slategray;
  text-align: right;
}

.historyLeft {
  display: flex;
  flex-direction: column;
  text-align: right;
}

.historyContent {
  flex: 1;
}

.historyTitle {
  font-weight: 400;
}

.historyText {
  font-size: 90%;
}

.historySmall {
  font-size: 70%;
  text-transform: uppercase;
}

.historyAmount {
  flex-grow: 1;
}

.historyAmount .primary {
  font-size: 100%;
}

.historyAmount .secondary {
  font-size: 80%;
}

.historyAmount .positive {
  color: #088;
}

.historyAmount .positive:before {
  content: "+";
}

.historyAmount .negative {
  color: #800;
}

.historyAmount .negative:before {
  color: #800;
  content: "-";
}
.icon {
  margin: 0 10px;
  text-align: center;
  width: 35px;
  font-size: 20px;
  border-radius: 50%;
  background: #ccc;
  color: #fff;
  padding-top: 4px;
  height: 30px;
}

.option {
  text-transform: uppercase;
  text-align: right;
  padding: 0.4em;
  font-size: 0.9em;
}

input[type="checkbox"],
input[type="radio"] {
  vertical-align: middle;
  position: relative;
  bottom: 1px;
}

input[type="radio"] {
  bottom: 2px;
}

.balance {
  text-align: center;
  padding-top: 2em;
}

/**
 pay html
*/
button.accept {
  background-color: #5757d2;
  border: 1px solid black;
  border-radius: 5px;
  margin: 1em 0;
  padding: 0.5em;
  font-weight: bold;
  color: white;
}
button.linky {
  background: none !important;
  border: none;
  padding: 0 !important;

  font-family: arial, sans-serif;
  color: #069;
  text-decoration: underline;
  cursor: pointer;
}

input.url {
  width: 25em;
}

button.accept:disabled {
  background-color: #dedbe8;
  border: 1px solid white;
  border-radius: 5px;
  margin: 1em 0;
  padding: 0.5em;
  font-weight: bold;
  color: #2c2c2c;
}

.errorbox {
  border: 2px solid #f5c6cb;
  border-radius: 0.25em;
  display: block;
  /* margin: 0.5em; */
  padding-left: 1em;
  padding-right: 1em;
  width: "100%";
  color: #721c24;
  background: #f8d7da;
}

.okaybox {
  border: 1px solid;
  display: inline-block;
  margin: 1em;
  padding: 1em;
  font-weight: bold;
  background: #00fa9a;
}

table.detailsTable td {
  text-align: right;
  border: 0px;
  border-bottom: 1px;
}

table.detailsTable td {
  border-bottom: 1px solid black;
}

table.detailsTable tr:last-child td {
  border-bottom: 0px;
}

table.detailsTable {
  border: 0px;
}

table.detailsTable.pending {
  color: gray;
}

.overflow {
  text-align: justify;
  position: relative;
  max-height: calc(1.2em * 2);
  overflow: hidden;
  padding-right: 1rem; /* space for ellipsis */
}
.overflow::before {
  position: absolute;
  content: "...";
  /*   inset-block-end: 0;
  inset-inline-end: 0; */
  bottom: 0;
  right: 0;
}
.overflow::after {
  content: "";
  position: absolute;
  /*   inset-inline-end: 0; */
  right: 0;
  width: 1rem;
  height: 1rem;
  background: white;
}

.button-success,
.button-destructive,
.button-warning,
.button-secondary {
  color: white;
  border-radius: 4px;
  text-shadow: 0 1px 1px rgba(0, 0, 0, 0.2);
}

.button-success {
  background: rgb(28, 184, 65);
}

.button-destructive {
  background: rgb(202, 60, 60);
}

.button-warning {
  background: rgb(223, 117, 20);
}

.button-secondary {
  background: rgb(66, 184, 221);
}

button {
  font-size: 120%;
  padding: 0.5em;
}
