// This file is generated automatically by `scripts/build/typings.js`. Please, don't change it.

import { areIntervalsOverlappingWithOptions } from 'date-fns/fp'
export default areIntervalsOverlappingWithOptions
