/*
 This file is part of GNU Taler
 (C) 2024-2025 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 *
 * @author Sebastian Javier Marchano (sebasjm)
 */

import { i18n, setupI18n } from "@gnu-taler/taler-util";
import * as tests from "../../tests/hook.js";
import { DefaultForm } from "../forms-ui.js";
import { acceptTos } from "./accept-tos.js";

setupI18n("en", {});

export const EmptyForm = tests.createExample(DefaultForm, {
  initial: {},
  design: acceptTos(i18n, {
    tos_url: "https://exchange.demo.taler.net/terms",
    provider_name: "Taler Operations AG",
    tos_version: "v1",
  }),
});

export default { title: "accept tos" };
