// flow-typed signature: 7d8521b06a61516b9b178233496d21bd
// flow-typed version: <<STUB>>/rollup-plugin-uglify_v^6.0.0/flow_v0.82.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   'rollup-plugin-uglify'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module 'rollup-plugin-uglify' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module 'rollup-plugin-uglify/transform' {
  declare module.exports: any;
}

// Filename aliases
declare module 'rollup-plugin-uglify/index' {
  declare module.exports: $Exports<'rollup-plugin-uglify'>;
}
declare module 'rollup-plugin-uglify/index.js' {
  declare module.exports: $Exports<'rollup-plugin-uglify'>;
}
declare module 'rollup-plugin-uglify/transform.js' {
  declare module.exports: $Exports<'rollup-plugin-uglify/transform'>;
}
