/*
 This file is part of GNU Taler
 (C) 2022 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 *
 * @author Sebastian Javier Marchano (sebasjm)
 */

import { TranslatedString } from "@gnu-taler/taler-util";
import * as tests from "../../tests/hook.js";
import { DefaultForm as TestedComponent } from "../forms-ui.js";
import { FormDesign, UIHandlerId } from "../forms-types.js";

export default {
  title: "Input Text Area",
};

export namespace Simplest {
  export interface Form {
    comment: string;
  }
}

type TargetObject = {
  comment: string;
};
const initial: TargetObject = {
  comment: "some initial comment",
};

const design: FormDesign = {
  type: "double-column",
  sections: [
    {
      title: "this is a simple form" as TranslatedString,
      fields: [
        {
          type: "textArea",
          label: "label of the field" as TranslatedString,
          id: "comment",
        },
      ],
    },
  ],
};

export const SimpleComment = tests.createExample(TestedComponent, {
  initial,
  design,
});
