
export interface StringsType {
  domain: string;
  lang: string;
  completeness: number;
  'plural_forms': string;
  locale_data: {
    messages: Record<string, unknown>;
  };
};
export const strings: Record<string,StringsType> = {};

strings['es'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Kyc completed": [
        "KYC completado"
      ],
      "You will be redirected to nowhere": [
        "Serias redirigido a ningun lugar"
      ],
      "Show debug info": [
        "Mostrar informacion de depuracion"
      ],
      "Internal error, please report.": [
        "Error interno, por favor reporte."
      ],
      "Customer identification": [
        "Identificacion de cliente"
      ],
      "Preferences": [
        "Preferencias"
      ],
      "Simple form": [
        "Formulario simple"
      ],
      "Comment": [
        "Comentario"
      ],
      "Resolution": [
        "Resolucion"
      ],
      "New state": [
        "Nuevo estado"
      ],
      "Frozen": [
        "Congelado"
      ],
      "Pending": [
        "Pendiente"
      ],
      "Normal": [
        "Normal"
      ],
      "New threshold": [
        "Nuevo limite"
      ],
      "Do you accept terms of service": [
        "Aceptas los terminos del servicio?"
      ],
      "Full Name": [
        "Nombre completo"
      ],
      "Birthdate": [
        "Fecha de nacimiento"
      ],
      "Are you a cross-border truck driver?": [
        "Sos un conductor de camiones limitrofe?"
      ],
      "Yes": [
        "Si"
      ],
      "No": [
        "No"
      ],
      "How much is in your pockets?": [
        "Cuanto dinero hay en tus bolsillos?"
      ],
      "Simple comment": [
        "Comentario simple"
      ],
      "Terms of Service": [
        "Terminos de servicio"
      ],
      "GLS onboarding": [
        "Incorporacion GLS"
      ],
      "Identification Form": [
        "Formulario de identificacion"
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        "El cliente deberá ser identificado al iniciar una relación comercial permanente o al concluir una transacción en efectivo que cumpla el umbral correspondiente."
      ],
      "Risk Profile AMLA": [
        "Perfil de riesgo AMLA"
      ],
      "Customer Profile": [
        "Perfil del cliente"
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        "La siguiente información debe referirse a las personas de las que proceden en última instancia los activos (por ejemplo, el beneficiario efectivo de los activos, el fundador o creador de un fideicomiso o fundación). Si el cliente es una entidad jurídica operativa o una sociedad, la información puede referirse a la propia entidad (no a la persona que ejerce el control), a menos que la entidad mantenga los activos en fideicomiso para un tercero."
      ],
      "Declaration of identity of the beneficial owner (A)": [
        "Declaración de identidad del beneficiario efectivo (A)"
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        "Constitución de la persona controladora de las personas jurídicas y sociedades operativas que no cotizan en bolsa (K)"
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        "para personas jurídicas operativas y sociedades que sean socios contratantes, así como análogamente para personas jurídicas operativas y sociedades que sean beneficiarios finales"
      ],
      "Foundations (as well as similar constructs) (S)": [
        "Fundaciones (así como construcciones similares) (S)"
      ],
      "Declaration for trusts (T)": [
        "Declaración de fideicomisos (T)"
      ],
      "Special Clarifications": [
        "Clarificaciones especiales"
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        "Cuando una relación o transacción comercial esté asociada a un mayor riesgo, parezca inusual o exista evidencia de que los activos son producto de un delito grave o un delito fiscal calificado, el miembro deberá realizar aclaraciones adicionales."
      ],
      "Insurance policies (I)": [
        "Pólizas de seguros (I)"
      ],
      "Information of life insurance policies with separately managed accounts/securities accounts (so-called insurance wrappers)": [
        "Información sobre pólizas de seguro de vida con cuentas de gestión separada/cuentas de valores (las denominadas \"envoltorios de seguros\")"
      ],
      "Name and birthdate": [
        "Nombre y fecha de nacimiento"
      ],
      "The form is too big for the server, try uploading smaller files.": [
        "El formulario es demasiado grande para el servidor, intente cargar archivos más pequeños."
      ],
      "The account was not found": [
        "No se encontro la cuenta"
      ],
      "Officer disabled or more recent decision was already submitted.": [
        "Oficial incapacitado o decisión más reciente ya fue presentada."
      ],
      "Debug information about the values in the form": [
        "Información de depuración sobre los valores en el formulario"
      ],
      "Cancel": [
        "Cancelar"
      ],
      "Submit": [
        "Enviar"
      ],
      "Mandatory identification requirements are satisfied": [
        "Se cumplen los requisitos de identificación obligatorios"
      ],
      "Process completed": [
        "Proceso terminado"
      ],
      "You can close this window now.": [
        "Puede cerrar esta ventana ahora."
      ],
      "Please, complete the following requirement:": [
        "Por favor, complete el siguiente requerimiento:"
      ],
      "Please, complete any of the following requirements:": [
        "Por favor, complete alguno de los siguientes requisitos:"
      ],
      "Start": [
        "Comenzar"
      ],
      "Form": [
        "Formulario"
      ],
      "Fill form": [
        "Llenar el formulario"
      ],
      "Trigger KYC balance": [
        "Iniciar balance KYC"
      ],
      "Trigger KYC Balance": [
        "Iniciar balance KYC"
      ],
      "Amount": [
        "Monto"
      ],
      "could not create token": [
        "no se pudo crear el token"
      ],
      "access denied": [
        "Acceso denegado"
      ],
      "not found": [
        "No encontrado"
      ],
      "conflict": [
        "Conflicto"
      ],
      "No kyc required": [
        "No se require KYC"
      ],
      "Access denied trying to test balance.": [
        "Acceso denegado intentando probar el balance."
      ],
      "Unavailable For Legal Reasons": [
        "No disponible por razones legales"
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        "Estas acciones activarán el saldo de KYC de la billetera por encima del umbral de 1000000; el intercambio debe estar configurado correctamente para activar el flujo de KYC deseado."
      ],
      "Trigger TOPS Terms of service": [
        "Iniciar TOPS Terms o service"
      ],
      "Trigger GLS onboarding": [
        "Iniciar GLS onboarding"
      ],
      "Trigger VQF 902.1": [
        "Iniciar VQF 902.1"
      ],
      "Trigger VQF 902.4": [
        "Iniciar VQF 902.4"
      ],
      "Trigger VQF 902.5": [
        "Iniciar VQF 902.5"
      ],
      "Trigger VQF 902.9": [
        "Iniciar VQF 902.9"
      ],
      "Trigger VQF 902.11": [
        "Iniciar VQF 902.11"
      ],
      "Trigger VQF 902.12": [
        "Iniciar VQF 902.12"
      ],
      "Trigger VQF 902.13": [
        "Iniciar VQF 902.13"
      ],
      "Trigger VQF 902.14": [
        "Iniciar VQF 902.14"
      ],
      "Trigger VQF 902.15": [
        "Iniciar VQF 902.15"
      ],
      "Challenger test": [
        "Probar challenger"
      ],
      "The URL is wrong": [
        "La URL es incorrecta"
      ],
      "Maybe the validation check expired.": [
        "Quizas la verificacion expiro."
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        ""
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 97
};

strings['en'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Kyc completed": [
        ""
      ],
      "You will be redirected to nowhere": [
        ""
      ],
      "Show debug info": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Customer identification": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Simple form": [
        ""
      ],
      "Comment": [
        ""
      ],
      "Resolution": [
        ""
      ],
      "New state": [
        ""
      ],
      "Frozen": [
        ""
      ],
      "Pending": [
        ""
      ],
      "Normal": [
        ""
      ],
      "New threshold": [
        ""
      ],
      "Do you accept terms of service": [
        ""
      ],
      "Full Name": [
        ""
      ],
      "Birthdate": [
        ""
      ],
      "Are you a cross-border truck driver?": [
        ""
      ],
      "Yes": [
        ""
      ],
      "No": [
        ""
      ],
      "How much is in your pockets?": [
        ""
      ],
      "Simple comment": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "GLS onboarding": [
        ""
      ],
      "Identification Form": [
        ""
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        ""
      ],
      "Risk Profile AMLA": [
        ""
      ],
      "Customer Profile": [
        ""
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        ""
      ],
      "Declaration of identity of the beneficial owner (A)": [
        ""
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        ""
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        ""
      ],
      "Foundations (as well as similar constructs) (S)": [
        ""
      ],
      "Declaration for trusts (T)": [
        ""
      ],
      "Special Clarifications": [
        ""
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        ""
      ],
      "Insurance policies (I)": [
        ""
      ],
      "Information of life insurance policies with separately managed accounts/securities accounts (so-called insurance wrappers)": [
        ""
      ],
      "Name and birthdate": [
        ""
      ],
      "The form is too big for the server, try uploading smaller files.": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Debug information about the values in the form": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Submit": [
        ""
      ],
      "Mandatory identification requirements are satisfied": [
        ""
      ],
      "Process completed": [
        ""
      ],
      "You can close this window now.": [
        ""
      ],
      "Please, complete the following requirement:": [
        ""
      ],
      "Please, complete any of the following requirements:": [
        ""
      ],
      "Start": [
        ""
      ],
      "Form": [
        ""
      ],
      "Fill form": [
        ""
      ],
      "Trigger KYC balance": [
        ""
      ],
      "Trigger KYC Balance": [
        ""
      ],
      "Amount": [
        ""
      ],
      "could not create token": [
        ""
      ],
      "access denied": [
        ""
      ],
      "not found": [
        ""
      ],
      "conflict": [
        ""
      ],
      "No kyc required": [
        ""
      ],
      "Access denied trying to test balance.": [
        ""
      ],
      "Unavailable For Legal Reasons": [
        ""
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        ""
      ],
      "Trigger TOPS Terms of service": [
        ""
      ],
      "Trigger GLS onboarding": [
        ""
      ],
      "Trigger VQF 902.1": [
        ""
      ],
      "Trigger VQF 902.4": [
        ""
      ],
      "Trigger VQF 902.5": [
        ""
      ],
      "Trigger VQF 902.9": [
        ""
      ],
      "Trigger VQF 902.11": [
        ""
      ],
      "Trigger VQF 902.12": [
        ""
      ],
      "Trigger VQF 902.13": [
        ""
      ],
      "Trigger VQF 902.14": [
        ""
      ],
      "Trigger VQF 902.15": [
        ""
      ],
      "Challenger test": [
        ""
      ],
      "The URL is wrong": [
        ""
      ],
      "Maybe the validation check expired.": [
        ""
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        ""
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 0
};

strings['de'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Kyc completed": [
        ""
      ],
      "You will be redirected to nowhere": [
        ""
      ],
      "Show debug info": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Customer identification": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Simple form": [
        ""
      ],
      "Comment": [
        ""
      ],
      "Resolution": [
        ""
      ],
      "New state": [
        ""
      ],
      "Frozen": [
        ""
      ],
      "Pending": [
        ""
      ],
      "Normal": [
        ""
      ],
      "New threshold": [
        ""
      ],
      "Do you accept terms of service": [
        ""
      ],
      "Full Name": [
        ""
      ],
      "Birthdate": [
        ""
      ],
      "Are you a cross-border truck driver?": [
        ""
      ],
      "Yes": [
        ""
      ],
      "No": [
        ""
      ],
      "How much is in your pockets?": [
        ""
      ],
      "Simple comment": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "GLS onboarding": [
        ""
      ],
      "Identification Form": [
        ""
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        ""
      ],
      "Risk Profile AMLA": [
        ""
      ],
      "Customer Profile": [
        ""
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        ""
      ],
      "Declaration of identity of the beneficial owner (A)": [
        ""
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        ""
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        ""
      ],
      "Foundations (as well as similar constructs) (S)": [
        ""
      ],
      "Declaration for trusts (T)": [
        ""
      ],
      "Special Clarifications": [
        ""
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        ""
      ],
      "Insurance policies (I)": [
        ""
      ],
      "Information of life insurance policies with separately managed accounts/securities accounts (so-called insurance wrappers)": [
        ""
      ],
      "Name and birthdate": [
        ""
      ],
      "The form is too big for the server, try uploading smaller files.": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Debug information about the values in the form": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Submit": [
        ""
      ],
      "Mandatory identification requirements are satisfied": [
        ""
      ],
      "Process completed": [
        ""
      ],
      "You can close this window now.": [
        ""
      ],
      "Please, complete the following requirement:": [
        ""
      ],
      "Please, complete any of the following requirements:": [
        ""
      ],
      "Start": [
        ""
      ],
      "Form": [
        ""
      ],
      "Fill form": [
        ""
      ],
      "Trigger KYC balance": [
        ""
      ],
      "Trigger KYC Balance": [
        ""
      ],
      "Amount": [
        ""
      ],
      "could not create token": [
        ""
      ],
      "access denied": [
        ""
      ],
      "not found": [
        ""
      ],
      "conflict": [
        ""
      ],
      "No kyc required": [
        ""
      ],
      "Access denied trying to test balance.": [
        ""
      ],
      "Unavailable For Legal Reasons": [
        ""
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        ""
      ],
      "Trigger TOPS Terms of service": [
        ""
      ],
      "Trigger GLS onboarding": [
        ""
      ],
      "Trigger VQF 902.1": [
        ""
      ],
      "Trigger VQF 902.4": [
        ""
      ],
      "Trigger VQF 902.5": [
        ""
      ],
      "Trigger VQF 902.9": [
        ""
      ],
      "Trigger VQF 902.11": [
        ""
      ],
      "Trigger VQF 902.12": [
        ""
      ],
      "Trigger VQF 902.13": [
        ""
      ],
      "Trigger VQF 902.14": [
        ""
      ],
      "Trigger VQF 902.15": [
        ""
      ],
      "Challenger test": [
        ""
      ],
      "The URL is wrong": [
        ""
      ],
      "Maybe the validation check expired.": [
        ""
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        ""
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 0
};

