// flow-typed signature: 128c4a691e1f78786a01e9aec17e241d
// flow-typed version: <<STUB>>/babel-cli_v^6.26.0/flow_v0.70.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   'babel-cli'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module 'babel-cli' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module 'babel-cli/bin/babel-doctor' {
  declare module.exports: any;
}

declare module 'babel-cli/bin/babel-external-helpers' {
  declare module.exports: any;
}

declare module 'babel-cli/bin/babel-node' {
  declare module.exports: any;
}

declare module 'babel-cli/bin/babel' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/_babel-node' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/babel-external-helpers' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/babel-node' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/babel/dir' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/babel/file' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/babel/index' {
  declare module.exports: any;
}

declare module 'babel-cli/lib/babel/util' {
  declare module.exports: any;
}

// Filename aliases
declare module 'babel-cli/bin/babel-doctor.js' {
  declare module.exports: $Exports<'babel-cli/bin/babel-doctor'>;
}
declare module 'babel-cli/bin/babel-external-helpers.js' {
  declare module.exports: $Exports<'babel-cli/bin/babel-external-helpers'>;
}
declare module 'babel-cli/bin/babel-node.js' {
  declare module.exports: $Exports<'babel-cli/bin/babel-node'>;
}
declare module 'babel-cli/bin/babel.js' {
  declare module.exports: $Exports<'babel-cli/bin/babel'>;
}
declare module 'babel-cli/index' {
  declare module.exports: $Exports<'babel-cli'>;
}
declare module 'babel-cli/index.js' {
  declare module.exports: $Exports<'babel-cli'>;
}
declare module 'babel-cli/lib/_babel-node.js' {
  declare module.exports: $Exports<'babel-cli/lib/_babel-node'>;
}
declare module 'babel-cli/lib/babel-external-helpers.js' {
  declare module.exports: $Exports<'babel-cli/lib/babel-external-helpers'>;
}
declare module 'babel-cli/lib/babel-node.js' {
  declare module.exports: $Exports<'babel-cli/lib/babel-node'>;
}
declare module 'babel-cli/lib/babel/dir.js' {
  declare module.exports: $Exports<'babel-cli/lib/babel/dir'>;
}
declare module 'babel-cli/lib/babel/file.js' {
  declare module.exports: $Exports<'babel-cli/lib/babel/file'>;
}
declare module 'babel-cli/lib/babel/index.js' {
  declare module.exports: $Exports<'babel-cli/lib/babel/index'>;
}
declare module 'babel-cli/lib/babel/util.js' {
  declare module.exports: $Exports<'babel-cli/lib/babel/util'>;
}
