import { default as tw_typo } from '@tailwindcss/typography';
import { default as tw_form } from '@tailwindcss/forms';

/** @type {import('tailwindcss').Config} */
export default {
  content: {
    relative: true,
    files: [
      "./src/**/*.{html,tsx}",
      "./node_modules/@gnu-taler/web-util/src/**/*.{html,tsx}"
    ],
  },
  theme: {
    extend: {},
  },
  plugins: [tw_typo, tw_form],
};
