/*
 This file is part of GNU Taler
 (C) 2019 GNUnet e.V.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

import test from "ava";
import { Duration } from "./time.js";

test("duration parsing", (t) => {
  const d1 = Duration.fromPrettyString("1h");
  t.deepEqual(d1.d_ms, 60 * 60 * 1000);

  const d2 = Duration.fromPrettyString("  2h 1s 3m");
  t.deepEqual(d2.d_ms, 2 * 60 * 60 * 1000 + 3 * 60 * 1000 + 1000);

  t.throws(() => {
    Duration.fromPrettyString("5g");
  });
  t.throws(() => {
    Duration.fromPrettyString("s");
  });
  t.throws(() => {
    Duration.fromPrettyString("s5");
  });
  t.throws(() => {
    Duration.fromPrettyString("5 5 s");
  });
});
