// flow-typed signature: 09d067c0cdb09defa76ef79dd3f8bdff
// flow-typed version: <<STUB>>/babel-jest_v^23.6.0/flow_v0.82.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   'babel-jest'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module 'babel-jest' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module 'babel-jest/build/index' {
  declare module.exports: any;
}

// Filename aliases
declare module 'babel-jest/build/index.js' {
  declare module.exports: $Exports<'babel-jest/build/index'>;
}
