// This file is generated automatically by `scripts/build/fp.js`. Please, don't change it.

import fn from '../../endOfMinute/index'
import convertToFP from '../_lib/convertToFP/index'

var endOfMinute = convertToFP(fn, 1)

export default endOfMinute
