/*
 This file is part of GNU Taler
 (C) 2020-2024 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 * Imports.
 */
import {
  Duration,
  j2s,
  NotificationType,
  succeedOrThrow,
  TalerMerchantInstanceHttpClient,
  TransactionMajorState,
  TransactionType,
  WalletNotification,
} from "@gnu-taler/taler-util";
import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
import {
  createSimpleTestkudosEnvironmentV3,
  withdrawViaBankV3,
} from "../harness/environments.js";
import { GlobalTestState } from "../harness/harness.js";

export async function runRefundTest(t: GlobalTestState) {
  // Set up test environment

  const {
    walletClient: wallet,
    bankClient,
    exchange,
    merchant,
    merchantAdminAccessToken,
  } = await createSimpleTestkudosEnvironmentV3(t);

  const notifs: WalletNotification[] = [];

  wallet.addNotificationListener((x) => {
    notifs.push(x);
  });

  const merchantClient = new TalerMerchantInstanceHttpClient(
    merchant.makeInstanceBaseUrl(),
  );

  // Withdraw digital cash into the wallet.

  const withdrawalRes = await withdrawViaBankV3(t, {
    walletClient: wallet,
    bankClient,
    exchange,
    amount: "TESTKUDOS:20",
  });

  await withdrawalRes.withdrawalFinishedCond;

  // Set up order.
  const orderResp = succeedOrThrow(
    await merchantClient.createOrder(merchantAdminAccessToken, {
      order: {
        summary: "Buy me!",
        amount: "TESTKUDOS:5",
        fulfillment_url: "taler://fulfillment-success/thx",
      },
      refund_delay: Duration.toTalerProtocolDuration(
        Duration.fromSpec({ minutes: 5 }),
      ),
    }),
  );

  let orderStatus = succeedOrThrow(
    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
  );

  t.assertTrue(orderStatus.order_status === "unpaid");

  // Make wallet pay for the order

  const r1 = await wallet.client.call(WalletApiOperation.PreparePayForUri, {
    talerPayUri: orderStatus.taler_pay_uri,
  });

  await wallet.client.call(WalletApiOperation.ConfirmPay, {
    transactionId: r1.transactionId,
  });

  // Check if payment was successful.

  orderStatus = succeedOrThrow(
    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
  );

  t.assertTrue(orderStatus.order_status === "paid");

  {
    const tx = await wallet.client.call(WalletApiOperation.GetTransactionById, {
      transactionId: r1.transactionId,
    });

    t.assertTrue(
      tx.type === TransactionType.Payment && tx.refundPending === undefined,
    );
  }

  const ref = succeedOrThrow(
    await merchantClient.addRefund(merchantAdminAccessToken, orderResp.order_id, {
      refund: "TESTKUDOS:5",
      reason: "foo",
    }),
  );

  console.log(ref);

  {
    const refundFinishedCond = wallet.waitForNotificationCond(
      (x) =>
        x.type === NotificationType.TransactionStateTransition &&
        x.transactionId === r1.transactionId &&
        x.newTxState.major === TransactionMajorState.Done,
    );
    await wallet.client.call(WalletApiOperation.StartRefundQuery, {
      transactionId: r1.transactionId,
    });
    await refundFinishedCond;
  }

  {
    const r2 = await wallet.client.call(WalletApiOperation.GetBalances, {});
    console.log(JSON.stringify(r2, undefined, 2));
  }

  {
    const r2 = await wallet.client.call(WalletApiOperation.GetTransactions, {});
    console.log(JSON.stringify(r2, undefined, 2));
  }

  {
    const tx = await wallet.client.call(WalletApiOperation.GetTransactionById, {
      transactionId: r1.transactionId,
    });

    console.log(j2s(tx));

    t.assertTrue(
      tx.type === TransactionType.Payment && tx.refundPending === undefined,
    );
  }

  //console.log(j2s(notifs));

  const refundDoneNotif = notifs.find(
    (x) =>
      x.type === NotificationType.TransactionStateTransition &&
      x.transactionId.startsWith("txn:refund:") &&
      x.newTxState.major === TransactionMajorState.Done,
  );

  t.assertTrue(!!refundDoneNotif);
}

runRefundTest.suites = ["wallet"];
