/**
 * @category Types
 * @summary Duration object
 *
 * @description
 * Duration object.
 *
 * @typedef {Object} Duration
 *
 * @property {number} [years]
 * @property {number} [months]
 * @property {number} [weeks]
 * @property {number} [days]
 * @property {number} [hours]
 * @property {number} [minutes]
 * @property {number} [seconds]
 */
const Duration = {}
module.exports = Duration
