export const strings: any = {};

strings["de"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=n != 1;",
        lang: "de",
      },
      Balance: ["Guthaben"],
      Backup: ["Backup"],
      "QR Reader and Taler URI": [""],
      Settings: ["Einstellungen"],
      Dev: ["Dev"],
      "%1$s": [""],
      "PENDING OPERATIONS": [""],
      Loading: ["Lädt Daten"],
      "Could not load backup providers": [""],
      "No backup providers configured": [""],
      "Add provider": [""],
      "Sync all backups": [""],
      "Sync now": [""],
      "Last synced": [""],
      "Not synced": [""],
      "Expires in": [""],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        "",
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [""],
      "See providers": [""],
      "Last backup": [""],
      "Back up": [""],
      "Provider fee": [""],
      "per year": [""],
      Extend: [""],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [""],
      old: [""],
      new: [""],
      fee: [""],
      storage: [""],
      "Remove provider": [""],
      "This provider has reported an error": [""],
      "There is conflict with another backup from %1$s": [""],
      "Backup is not readable": [""],
      "Unknown backup problem: %1$s": [""],
      "service paid": [""],
      "Backup valid until": [""],
      Cancel: ["Abbrechen"],
      "Open reserve page": ["Seite der Reserve aufrufen"],
      "Open pay page": ["Seite für Zahlungen aufrufen"],
      "Open refund page": ["Seite für Rückerstattungen aufrufen"],
      "Open tip page": ["Seite der Aufwandsentschädigungen aufrufen"],
      "Open withdraw page": ["Abhebeseite öffnen"],
      "Get digital cash": ["Digitales Bargeld abheben"],
      "Could not load balance page": ["Konnte die Umsatzanzeige nicht laden"],
      Add: [""],
      "Send %1$s": [""],
      "Taler Action": [""],
      "This page has pay action.": [""],
      "This page has a withdrawal action.": [""],
      "This page has a tip action.": [""],
      "This page has a notify reserve action.": [""],
      Notify: [""],
      "This page has a refund action.": [""],
      "This page has a malformed taler uri.": [""],
      Dismiss: [""],
      "this popup is being closed and you are being redirected to %1$s": [""],
      "Could not load purchase proposal details": [
        "Konnte die Umsatzanzeige nicht laden",
      ],
      "Order Id": [""],
      Summary: [""],
      Amount: ["Betrag"],
      "Merchant name": [""],
      "Merchant jurisdiction": [""],
      "Merchant address": [""],
      "Merchant logo": [""],
      "Merchant website": [""],
      "Merchant email": [""],
      "Merchant public key": [""],
      "Delivery date": [""],
      "Delivery location": [""],
      Products: [""],
      "Created at": [""],
      "Refund deadline": [""],
      "Auto refund": [""],
      "Pay deadline": [""],
      "Fulfillment URL": [""],
      "Fulfillment message": [""],
      "Max deposit fee": [""],
      "Max fee": [""],
      "Minimum age": [""],
      "Wire fee amortization": [""],
      Auditors: [""],
      Exchanges: ["Exchange"],
      "Bank account": [""],
      "Bitcoin address": [""],
      IBAN: [""],
      "Could not load deposit status": ["Konnte die Umsatzanzeige nicht laden"],
      "Digital cash deposit": [""],
      Cost: [""],
      Fee: [""],
      "To be received": [""],
      "Send &nbsp; %1$s": [""],
      "Bitcoin transfer details": [""],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [""],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [""],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [""],
      Account: [""],
      "Bank host": [""],
      "Bank transfer details": [""],
      Subject: ["Verwendungszweck"],
      "Receiver name": [""],
      "Could not load the transaction information": [""],
      "There was an error trying to complete the transaction": [""],
      "This transaction is not completed": [""],
      Send: [""],
      Retry: ["Erneut versuchen"],
      Forget: [""],
      "Caution!": [""],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [""],
      Confirm: ["Bestätigen"],
      Withdrawal: ["Abheben"],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [""],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [""],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [""],
      Details: [""],
      Payment: ["Zahlung"],
      Refunds: [""],
      "%1$s %2$s on %3$s": [
        "%1$s\n               möchte einen Vertrag über %2$s\n               mit Ihnen abschließen.",
      ],
      "Merchant created a refund for this order but was not automatically picked up.":
        [""],
      Offer: [""],
      Accept: [""],
      Merchant: [""],
      "Invoice ID": [""],
      Deposit: [""],
      Refresh: [""],
      Tip: [""],
      Refund: [""],
      "Original order ID": [""],
      "Purchase summary": [""],
      copy: [""],
      "hide qr": [""],
      "show qr": [""],
      Credit: [""],
      Invoice: [""],
      Exchange: ["Exchange"],
      URI: [""],
      Debit: [""],
      Transfer: [""],
      Country: ["Betrag"],
      "Address lines": [""],
      "Building number": [""],
      "Building name": [""],
      Street: [""],
      "Post code": [""],
      "Town location": [""],
      Town: [""],
      District: [""],
      "Country subdivision": [""],
      Date: [""],
      "Transaction fees": [""],
      Total: [""],
      Withdraw: ["Abheben"],
      Price: [""],
      Refunded: [""],
      Delivery: [""],
      "Total transfer": ["Insgesamt abgehoben"],
      "Could not load pay status": [""],
      "Digital cash payment": [""],
      Purchase: [""],
      Receipt: [""],
      "Valid until": [""],
      "List of products": [""],
      free: [""],
      "Already paid, you are going to be redirected to %1$s": [""],
      "Already paid": [""],
      "Already claimed": [""],
      "Pay with a mobile phone": [""],
      "Hide QR": [""],
      "Scan the QR code or &nbsp; %1$s": [""],
      "Pay &nbsp; %1$s": [""],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [""],
      "Your current balance is not enough.": [
        "Es gibt kein Guthaben anzuzeigen.",
      ],
      "Merchant message": [""],
      "Could not load refund status": ["Konnte die Umsatzanzeige nicht laden"],
      "Digital cash refund": [""],
      "You&apos;ve ignored the tip.": [""],
      "The refund is in progress.": [""],
      "Total to refund": ["Insgesamt abgehoben"],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Der Händler %1$s bietet Ihnen eine Aufwandsentschädigung von %2$s durch den Exchange %3$s",
      ],
      "Order amount": [""],
      "Already refunded": [""],
      "Refund offered": [""],
      "Accept &nbsp; %1$s": [""],
      "Could not load tip status": ["Konnte die Umsatzanzeige nicht laden"],
      "Digital cash tip": [""],
      "The merchant is offering you a tip": [
        "Der Händler %1$s bietet Ihnen eine Aufwandsentschädigung von %2$s durch den Exchange %3$s",
      ],
      "Merchant URL": [""],
      "Receive &nbsp; %1$s": [""],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [""],
      "Select one option": [""],
      "Could not load": ["Konnte die Umsatzanzeige nicht laden"],
      "Show terms of service": [""],
      "I accept the exchange terms of service": [""],
      "Exchange doesn&apos;t have terms of service": [""],
      "Review exchange terms of service": [""],
      "Review new version of terms of service": [""],
      "The exchange reply with a empty terms of service": [""],
      "Download Terms of Service": [""],
      "Hide terms of service": [""],
      "Could not load exchange fees": ["Konnte die Umsatzanzeige nicht laden"],
      Close: [""],
      "could not find any exchange": ["Konnte die Umsatzanzeige nicht laden"],
      "could not find any exchange for the currency %1$s": [""],
      "Service fee description": [""],
      "Select %1$s exchange": [""],
      Reset: [""],
      "Use this exchange": [""],
      "Doesn&apos;t have auditors": [""],
      currency: [""],
      Operations: [""],
      Deposits: ["%1$s zahlen"],
      Denomination: [""],
      Until: [""],
      Withdrawals: ["Abheben"],
      Currency: [""],
      "Coin operations": [""],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [""],
      "Transfer operations": [""],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [""],
      Operation: [""],
      "Wallet operations": [""],
      Feature: [""],
      "Could not get the info from the URI": [""],
      "Could not get info of withdrawal": [""],
      "Digital cash withdrawal": [""],
      "Could not finish the withdrawal operation": [""],
      "Age restriction": [""],
      "Withdraw &nbsp; %1$s": ["Abheben bei %1$s"],
      "Withdraw to a mobile phone": [""],
      "Digital invoice": [""],
      "Could not finish the invoice creation": [""],
      Create: [""],
      "Could not finish the payment operation": [""],
      "Digital cash transfer": [""],
      "Could not finish the transfer creation": [""],
      "Could not finish the pickup operation": [""],
      "Manual Withdrawal for %1$s": ["Manuelles Abheben"],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [""],
      "No exchange found for %1$s": [""],
      "Add Exchange": [""],
      "No exchange configured": [""],
      "Can&apos;t create the reserve": [""],
      "Start withdrawal": ["Abhebung beginnen"],
      "Could not load deposit balance": [
        "Konnte die Umsatzanzeige nicht laden",
      ],
      "A currency or an amount should be indicated": [""],
      "There is no enough balance to make a deposit for currency %1$s": [""],
      "Send %1$s to your account": [""],
      "There is no account to make a deposit for currency %1$s": [""],
      "Add account": [""],
      "Select account": [""],
      "Add another account": [""],
      "Deposit fee": [""],
      "Total deposit": [""],
      "Deposit&nbsp;%1$s %2$s": ["Einlösen %1$s %2$s"],
      "Add bank account for %1$s": [""],
      "Enter the URL of an exchange you trust.": [""],
      "Unable add this account": [""],
      "Select account type": [""],
      "Review terms of service": [""],
      "Exchange URL": [""],
      "Add exchange": [""],
      "Add new exchange": [""],
      "Add exchange for %1$s": [""],
      "An exchange has been found! Review the information and click next": [""],
      "This exchange doesn&apos;t match the expected currency %1$s": [""],
      "Unable to verify this exchange": [""],
      "Unable to add this exchange": [""],
      loading: [""],
      Version: [""],
      Next: [""],
      "Waiting for confirmation": [""],
      PENDING: [""],
      "Could not load the list of transactions": [""],
      "Your transaction history is empty for this currency.": [""],
      "Add backup provider": [""],
      "Could not get provider information": [""],
      "Backup providers may charge for their service": [""],
      URL: [""],
      Name: [""],
      "Provider URL": [""],
      "Please review and accept this provider&apos;s terms of service": [""],
      Pricing: [""],
      "free of charge": [""],
      "%1$s per year of service": [""],
      Storage: [""],
      "%1$s megabytes of storage per year of service": [""],
      "Accept terms of service": [""],
      "Could not parse the payto URI": [""],
      "Please check the uri": [""],
      "Exchange is ready for withdrawal": [""],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [""],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [""],
      "Cancel withdrawal": [""],
      "Could not toggle auto-open": ["Konnte die Umsatzanzeige nicht laden"],
      "Could not toggle clipboard": ["Konnte die Umsatzanzeige nicht laden"],
      Navigator: [""],
      "Automatically open wallet based on page content": [""],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [""],
      "Automatically check clipboard for Taler URI": [""],
      Trust: [""],
      "No exchange yet": [""],
      "Term of Service": [""],
      ok: [""],
      changed: [""],
      "not accepted": [""],
      "unknown (exchange status should be updated)": [""],
      "Add an exchange": [""],
      Troubleshooting: [""],
      "Developer mode": [""],
      "More options and information useful for debugging": [""],
      Display: [""],
      "Current Language": [""],
      "Wallet Core": [""],
      "Web Extension": [""],
      "Exchange compatibility": [""],
      "Merchant compatibility": [""],
      "Bank compatibility": [""],
      "Browser Extension Installed!": [""],
      "You can open the GNU Taler Wallet using the combination %1$s .": [""],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [""],
      "Click the puzzle icon": [""],
      "Search for GNU Taler Wallet": [""],
      "Click the pin icon": [""],
      Permissions: [""],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [""],
      "Next Steps": [""],
      "Try the demo": [""],
      "Learn how to top up your wallet balance": [""],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "Die Diagnostik ist abgeschlossen. Es war keine Kommunikation mit dem Wallet-Backend möglich.",
      ],
      "Problems detected:": ["Ein Problem wurde festgestellt:"],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [
          "Bitte prüfen Sie ihre %1$s Einstellungen, für die Sie IndexedDB verwenden (preference name %2$s prüfen).",
        ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [
          "Die Datenbank des Wallets ist veraltet. Aktuell wird jedoch keine Migration auf eine neue Version unterstützt. Bitte wählen Sie %1$s zum Zurücksetzen der Wallet-Datenbank.",
        ],
      "Running diagnostics": ["Diagnostik wird durchgeführt"],
      "Debug tools": ["Debugging-Tools"],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [""],
      reset: ["zurücksetzen"],
      "TESTING: This may delete all your coin, proceed with caution": [""],
      "run gc": [""],
      "import database": [""],
      "export database": [""],
      "Database exported at %1$s %2$s to download": [""],
      Coins: [""],
      "Pending operations": [""],
      "usable coins": [""],
      id: [""],
      denom: [""],
      value: [""],
      status: [""],
      "from refresh?": [""],
      "age key count": [""],
      "spent coins": [""],
      "click to show": [""],
      "Scan a QR code or enter taler:// URI below": [""],
      Open: [""],
      "URI is not valid. Taler URI should start with `taler://`": [""],
      "Try another": [""],
      "Could not load list of exchange": [
        "Konnte die Umsatzanzeige nicht laden",
      ],
      "Choose a currency to proceed or add another exchange": [""],
      "Known currencies": [""],
      "Specify the amount and the origin": [""],
      "Change currency": [""],
      "Use previous origins:": [""],
      "Or specify the origin of the money": [""],
      "Specify the origin of the money": [""],
      "From my bank account": [""],
      "From another wallet": [""],
      "currency not provided": [""],
      "Specify the amount and the destination": [""],
      "Use previous destinations:": [""],
      "Or specify the destination of the money": [""],
      "Specify the destination of the money": [""],
      "To my bank account": [""],
      "To another wallet": [""],
      "Could not load backup recovery information": [
        "Konnte die Umsatzanzeige nicht laden",
      ],
      "Digital wallet recovery": [""],
      "Import backup, show info": [""],
      "All done, your transaction is in progress": [""],
      Edit: [""],
      "Could not load the list of known exchanges": [""],
    },
  },
};

strings["es"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=n != 1;",
        lang: "es",
      },
      Balance: ["Balance"],
      Backup: ["Copia de seguridad"],
      "QR Reader and Taler URI": ["Lector QR y Taler URI"],
      Settings: ["Configuración"],
      Dev: ["Dev"],
      "%1$s": ["%1$s"],
      "PENDING OPERATIONS": ["OPERACIONES PENDIENTES"],
      Loading: ["Cargando"],
      "Could not load backup providers": [
        "No se pudo cargar los proveedores de copias de seguridad",
      ],
      "No backup providers configured": [
        "No hay proveedores de copias de seguridad configurados",
      ],
      "Add provider": ["Agregar proveedor"],
      "Sync all backups": ["Sincronizar todas las copias de seguridad"],
      "Sync now": ["Sincronizar ahora"],
      "Last synced": ["Ultima vez sincronizado"],
      "Not synced": ["No sincronizado"],
      "Expires in": ["Expira en"],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        'Hubo un error cargando los detalles del proveedor para "%1$s"',
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        'No hay proveedor conocido con la URL "%1$s".',
      ],
      "See providers": ["Ver proveedores"],
      "Last backup": ["Última copia de seguridad"],
      "Back up": ["Copia de seguridad"],
      "Provider fee": ["Tarifa del proveedor"],
      "per year": ["por año"],
      Extend: ["Extender"],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [
          "los términos han cambiado, extender el servicio implicará aceptar los nuevos términos de servicio",
        ],
      old: ["viejo"],
      new: ["nuevo"],
      fee: ["tarifa"],
      storage: ["almacenamiento"],
      "Remove provider": ["Eliminar proveedor"],
      "This provider has reported an error": [
        "Este proveedor ha reportado un error",
      ],
      "There is conflict with another backup from %1$s": [
        "Hay un conflicto con otra copia de seguridad de %1$s",
      ],
      "Backup is not readable": ["La copia de seguridad no es legible"],
      "Unknown backup problem: %1$s": [
        "Problema de copia de seguridad desconocido: %1$s",
      ],
      "service paid": ["servicio pagado"],
      "Backup valid until": ["Copia de seguridad válida hasta"],
      Cancel: ["Cancelar"],
      "Open reserve page": ["Abrir página de reserva"],
      "Open pay page": ["Abrir página de pago"],
      "Open refund page": ["Abrir página de devolución"],
      "Open tip page": ["Abrir página de propina"],
      "Open withdraw page": ["Abrir página de retirada"],
      "Get digital cash": ["Retirar dinero digital"],
      "Could not load balance page": ["No se pudo cargar la página"],
      Add: ["Agregar"],
      "Send %1$s": ["Envíar %1$s"],
      "Taler Action": ["Acción Taler"],
      "This page has pay action.": ["Esta página tiene una acción de pago."],
      "This page has a withdrawal action.": [
        "Esta página tiene una acción de retirada.",
      ],
      "This page has a tip action.": [
        "Esta página tiene una acción de propina.",
      ],
      "This page has a notify reserve action.": [
        "Esta página tiene una acción de notificación de reserva.",
      ],
      Notify: ["Notificar"],
      "This page has a refund action.": [
        "Esta página tiene una acción de devolución.",
      ],
      "This page has a malformed taler uri.": [
        "Esta página tiene una URI de Taler malformada.",
      ],
      Dismiss: ["Descartar"],
      "this popup is being closed and you are being redirected to %1$s": [
        "Este popup está siendo cerrado y estás siendo redirigido a %1$s",
      ],
      "Could not load purchase proposal details": [
        "No se pudo cargar el detalle de la propuesta",
      ],
      "Order Id": ["Id de orden"],
      Summary: ["Resumen"],
      Amount: ["Cantidad"],
      "Merchant name": ["Comerciante"],
      "Merchant jurisdiction": ["Jurisdicción"],
      "Merchant address": ["Dirección del comerciante"],
      "Merchant logo": ["Logo"],
      "Merchant website": ["Siti web"],
      "Merchant email": ["Correo electrónico"],
      "Merchant public key": ["Clave pública"],
      "Delivery date": ["Fecha de entrega"],
      "Delivery location": ["Ubicación de entrega"],
      Products: ["Productos"],
      "Created at": ["Creado en"],
      "Refund deadline": ["Plazo de devolución"],
      "Auto refund": ["Devolución automática"],
      "Pay deadline": ["Plazo de pago"],
      "Fulfillment URL": ["URL de éxito"],
      "Fulfillment message": ["Mensaje de éxito"],
      "Max deposit fee": ["Máxima comisión de depósito"],
      "Max fee": ["Máxima comisión"],
      "Minimum age": ["Edad mínima"],
      "Wire fee amortization": ["Amortización de comisión de transferencia"],
      Auditors: ["Auditores"],
      Exchanges: ["Exchanges"],
      "Bank account": ["Cuenta del banco"],
      "Bitcoin address": ["Dirección de Bitcoin"],
      IBAN: ["IBAN"],
      "Could not load deposit status": [
        "No se pudo cargar el estado del depósito",
      ],
      "Digital cash deposit": ["Depósito de dinero digital"],
      Cost: ["Costo"],
      Fee: ["Comisión"],
      "To be received": ["A recibir"],
      "Send &nbsp; %1$s": ["Envíar %1$s"],
      "Bitcoin transfer details": ["Detalle de transferencia Bitcoin"],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [
          "El exchange necesita una transacción con 3 salidas, una salida es hacia la cuenta del exchange y las otras dos son direcciones segwit falsas para metadata con el monto mínimo.",
        ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [
          'En la billetera bitcoincore usar el botón "Agregar destinatario" para agregar dos destinatarios y copiar las direcciones y montos',
        ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [
          "Asegurarse de que el monto muestre %1$s BTC, sino tendrá que cambiar la unidad a BTC",
        ],
      Account: ["Cuenta"],
      "Bank host": ["Banco anfitrión"],
      "Bank transfer details": ["Detalle de transferencia bancaria"],
      Subject: ["Asunto"],
      "Receiver name": ["Nombre del receptor"],
      "Could not load the transaction information": [
        "No se pudo cargar información de la transacción",
      ],
      "There was an error trying to complete the transaction": [
        "Hubo un error intentando completar la transacción",
      ],
      "This transaction is not completed": [
        "Esta transacción no está completada",
      ],
      Send: ["Enviar"],
      Retry: ["Reintentar"],
      Forget: ["Olvidar"],
      "Caution!": ["Cuidado!"],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [
          "Si tú ya has transferido dinero al exchange, perderás la oportunidad de recibir las monedas desde este.",
        ],
      Confirm: ["Confirmar"],
      Withdrawal: ["Retirada"],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [
          "Asegúrate de usar el asunto correcto, de lo contrario el dinero no llegará a esta billetera.",
        ],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [
          "El banco todavía no confirmó la transferencia. Ir a %1$s %2$s y verificar que no hay pasos pendientes.",
        ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [
          "El banco confirmó la transferencia. Esperando que el exchange envíe las monedas",
        ],
      Details: ["Detalles"],
      Payment: ["Pago"],
      Refunds: ["Devoluciones"],
      "%1$s %2$s on %3$s": ["%1$s %2$s en %3$s"],
      "Merchant created a refund for this order but was not automatically picked up.":
        [
          "El comerciante creó una devolución para esta orden pero no fue recogida automáticamente.",
        ],
      Offer: ["Oferta"],
      Accept: ["Aceptar"],
      Merchant: ["Comerciante"],
      "Invoice ID": ["Id de factura"],
      Deposit: ["Depósito"],
      Refresh: ["Actualizar"],
      Tip: ["Propina"],
      Refund: ["Devolución"],
      "Original order ID": ["Id de orden original"],
      "Purchase summary": ["Resumen de compra"],
      copy: ["Copiar"],
      "hide qr": ["Esconder QR"],
      "show qr": ["Mostrar QR"],
      Credit: ["Crédito"],
      Invoice: ["Factura"],
      Exchange: ["Exchange"],
      URI: ["URI"],
      Debit: ["Débito"],
      Transfer: ["Transferencia"],
      Country: ["País"],
      "Address lines": ["Detalle de dirección"],
      "Building number": ["Número de edificio"],
      "Building name": ["Nombre de edificio"],
      Street: ["Calle"],
      "Post code": ["Código postal"],
      "Town location": ["Ubicación de la ciudad"],
      Town: ["Ciudad"],
      District: ["Distrito"],
      "Country subdivision": ["Subdivisión de país"],
      Date: ["Fecha"],
      "Transaction fees": ["Comisiones de transacción"],
      Total: ["Total"],
      Withdraw: ["Retirar"],
      Price: ["Precio"],
      Refunded: ["Devuelto"],
      Delivery: ["Entrega"],
      "Total transfer": ["Total transferido"],
      "Could not load pay status": ["No se pudo cargar el estado del pago"],
      "Digital cash payment": ["Pago con dinero digital"],
      Purchase: ["Compra"],
      Receipt: ["Recibo"],
      "Valid until": ["Válido hasta"],
      "List of products": ["Lista de productos"],
      free: ["Gratis"],
      "Already paid, you are going to be redirected to %1$s": [
        "Ya pagado, estás siendo dirigido a %1$s",
      ],
      "Already paid": ["Ya pagado"],
      "Already claimed": ["Ya reclamado"],
      "Pay with a mobile phone": ["Pagar con un teléfono móbil"],
      "Hide QR": ["Esconder QR"],
      "Scan the QR code or &nbsp; %1$s": ["Escanear el código QR o %1$s"],
      "Pay &nbsp; %1$s": ["Pagar %1$s"],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "No hay balance para esta divisa. Extraer dinero digital primero.",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [
          "No se encontraron suficientes monedas para pagar. Incluso si tuviera suficiente %1$s algunas restricciones se podrían aplicar.",
        ],
      "Your current balance is not enough.": ["Tu balance no es suficiente."],
      "Merchant message": ["Mensaje del comerciante"],
      "Could not load refund status": [
        "No se pudo cargar el estado de la devolución",
      ],
      "Digital cash refund": ["Devolución de dinero digital"],
      "You&apos;ve ignored the tip.": ["Has ignorado la propina."],
      "The refund is in progress.": [
        "El proceso de devolución está en progreso.",
      ],
      "Total to refund": ["Total para devolver"],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        'El comerciante "%1$s" te está ofreciendo una devolución.',
      ],
      "Order amount": ["Monto de la orden"],
      "Already refunded": ["Ya devuelto"],
      "Refund offered": ["Devolución ofrecida"],
      "Accept &nbsp; %1$s": ["Aceptar %1$s"],
      "Could not load tip status": [
        "No se pudo cargar el estado de la propina",
      ],
      "Digital cash tip": ["Propina con dinero digital"],
      "The merchant is offering you a tip": [
        "El comerciante te ofrece una propina",
      ],
      "Merchant URL": ["URL del comerciante"],
      "Receive &nbsp; %1$s": ["Recibir %1$s"],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [
          "Propina de %1$s aceptada. Revisa tu lista de transacciones para más detalle.",
        ],
      "Select one option": ["Seleccione una opción"],
      "Could not load": ["No se pudo cargar"],
      "Show terms of service": ["Mostrar términos de servicio"],
      "I accept the exchange terms of service": [
        "Yo acepto los términos de servicio del exchange",
      ],
      "Exchange doesn&apos;t have terms of service": [
        "El exchange no tiene los términos de servicio",
      ],
      "Review exchange terms of service": ["Revisar los términos de servicio"],
      "Review new version of terms of service": [
        "Revisar los nuevos términos de servicio",
      ],
      "The exchange reply with a empty terms of service": [
        "El exchange respondió con unos términos de servicio vacíos",
      ],
      "Download Terms of Service": ["Descargar los términos de servicio"],
      "Hide terms of service": ["Esconder los términos de servicio"],
      "Could not load exchange fees": [
        "No se pudo cargar la comisión del exchange",
      ],
      Close: ["Cerrar"],
      "could not find any exchange": ["No se pudo encontrar ningún exchange"],
      "could not find any exchange for the currency %1$s": [
        "No se pudo encontrar ningún exchange para la divisa %1$s",
      ],
      "Service fee description": ["Descripción de comisión de servicio"],
      "Select %1$s exchange": ["Seleccionar exchange %1$s"],
      Reset: ["Reiniciar"],
      "Use this exchange": ["Usar este exchange"],
      "Doesn&apos;t have auditors": ["No tiene auditores"],
      currency: ["Divisa"],
      Operations: ["Operaciones"],
      Deposits: ["Depósitos"],
      Denomination: ["Operaciones"],
      Until: ["Hasta"],
      Withdrawals: ["Retiradas"],
      Currency: ["Divisa"],
      "Coin operations": ["Operaciones de moneda"],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [
          "Toda operación en esta sección puede ser diferente por valor de denominación y es válida por un período. El exchange cobrará el monto indicado cada vez que una es usada en dicha operación.",
        ],
      "Transfer operations": ["Operaciones de transferencia"],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [
          "Toda operación en esta sección puede ser diferente por tipo de transacción y es válida por un período. El exchange cobrará el monto indicado cada vez que se haga una transferencia.",
        ],
      Operation: ["Operación"],
      "Wallet operations": ["Operaciones de billetera"],
      Feature: ["Característica"],
      "Could not get the info from the URI": [
        "No se pudo obtener la información desde la URI",
      ],
      "Could not get info of withdrawal": [
        "No se pudo obtener la información de retiro",
      ],
      "Digital cash withdrawal": ["Retirada de dinero digital"],
      "Could not finish the withdrawal operation": [
        "No se pudo completar la operación de retirada",
      ],
      "Age restriction": ["Restricción etaria"],
      "Withdraw &nbsp; %1$s": ["Retirar %1$s"],
      "Withdraw to a mobile phone": ["Retirar con un teléfono móvil"],
      "Digital invoice": ["Factura digital"],
      "Could not finish the invoice creation": [
        "No se pudo completar la creación de la factura",
      ],
      Create: ["Crear"],
      "Could not finish the payment operation": [
        "No se pudo completar la operación de pago",
      ],
      "Digital cash transfer": ["Transferencia de dinero digital"],
      "Could not finish the transfer creation": [
        "No se pudo completar la operación de creación de transferencia",
      ],
      "Could not finish the pickup operation": [
        "No se pudo completar la operación de recolección",
      ],
      "Manual Withdrawal for %1$s": ["Retirada Manual para %1$s"],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [
          "Elija un exchange desde donde las monedas serán retiradas. El exchange enviará las monedas a esta billetera después de recibir una transferencia bancaria con el asunto correcto.",
        ],
      "No exchange found for %1$s": ["No se encontró exchange para %1$s"],
      "Add Exchange": ["Agregar Exchange"],
      "No exchange configured": ["Sin exchange configurado"],
      "Can&apos;t create the reserve": ["No se pudo crear una reserva"],
      "Start withdrawal": ["Comenzar la retirada"],
      "Could not load deposit balance": [
        "No se pudo cargar el balance de depósito",
      ],
      "A currency or an amount should be indicated": [
        "Se debería especificar una divisa o un monto",
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        "No hay suficiente balance para hacer un depósito para la divisa %1$s",
      ],
      "Send %1$s to your account": ["Enviar %1$s a tu cuenta"],
      "There is no account to make a deposit for currency %1$s": [
        "No hay una cuenta para hacer un depósito para la divisa %1$s",
      ],
      "Add account": ["Agregar cuenta"],
      "Select account": ["Seleccionar cuenta"],
      "Add another account": ["Agregar otra cuenta"],
      "Deposit fee": ["Comisión de depósito"],
      "Total deposit": ["Depósito total"],
      "Deposit&nbsp;%1$s %2$s": ["Depositar %1$s %2$s"],
      "Add bank account for %1$s": ["Agregar cuenta de banco para %1$s"],
      "Enter the URL of an exchange you trust.": [
        "Ingresar la URL de un exchange en el que confíes.",
      ],
      "Unable add this account": ["No fue posible agregar esta cuenta"],
      "Select account type": ["Seleccione un tipo de cuenta"],
      "Review terms of service": ["Revisar los términos de servicio"],
      "Exchange URL": ["Exchange URL"],
      "Add exchange": ["Agregar exchange"],
      "Add new exchange": ["Agregar nuevo exchange"],
      "Add exchange for %1$s": ["Agregar exchange para %1$s"],
      "An exchange has been found! Review the information and click next": [
        "Un exchange ha sido encontrado! Revisa la información y haz clic en siguiente",
      ],
      "This exchange doesn&apos;t match the expected currency %1$s": [
        "Este exchange no coincide con la divisa %1$s esperada",
      ],
      "Unable to verify this exchange": [
        "No fue posible verificar este exchange",
      ],
      "Unable to add this exchange": ["No fue posible agregar este exchange"],
      loading: ["cargando"],
      Version: ["Versión"],
      Next: ["Siguiente"],
      "Waiting for confirmation": ["Esperando confirmación"],
      PENDING: ["PENDIENTE"],
      "Could not load the list of transactions": [
        "No se pudo cargar la lista de transacciones",
      ],
      "Your transaction history is empty for this currency.": [
        "No hay historial para esta divisa.",
      ],
      "Add backup provider": ["Agregar proveedor de copias de seguridad"],
      "Could not get provider information": [
        "No se pudo conseguir la información del proveedor",
      ],
      "Backup providers may charge for their service": [
        "Los proveedores de copias de seguridad pueden cobrarte por su servicio",
      ],
      URL: ["URL"],
      Name: ["Nombre"],
      "Provider URL": ["URL del proveedor"],
      "Please review and accept this provider&apos;s terms of service": [
        "Por favor revisa y acepta los términos de servicio del proveedor",
      ],
      Pricing: ["Precios"],
      "free of charge": ["Gratis"],
      "%1$s per year of service": ["%1$s por año de servicio"],
      Storage: ["Alamcenamiento"],
      "%1$s megabytes of storage per year of service": [
        "%1$s megabytes de almacenamiento por año de servicio",
      ],
      "Accept terms of service": ["Aceptar los términos de servicio"],
      "Could not parse the payto URI": [
        "No se pudo obtener la información de la URI payto",
      ],
      "Please check the uri": ["Revisar la URI"],
      "Exchange is ready for withdrawal": [
        "El exchange está listo para la retirada",
      ],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [
          "Para completar el proceso necesitas transferir %1$s %2$s a la cuenta bancaria del exchange",
        ],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [
          "Alternativamente, también puedes escanear el código QR o abrir %1$s si tienes una App bancaria instalada que soporta RFC 8905",
        ],
      "Cancel withdrawal": ["Cancelar retirada"],
      "Could not toggle auto-open": ["No se pudo cambiar el auto-open"],
      "Could not toggle clipboard": ["No se pudo cambiar portapapeles"],
      Navigator: ["Navegador"],
      "Automatically open wallet based on page content": [
        "Abrir automáticamente la billetera basada en el contenido de la página",
      ],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [
          "Habilitar la opción de debajo, hará que el uso de la billetera sea mas rápido, pero requiere más permisos de tu navegador.",
        ],
      "Automatically check clipboard for Taler URI": [
        "Revisar el portapapeles automáticamente por Taler URI",
      ],
      Trust: ["Confianza"],
      "No exchange yet": ["No hay exchanges todavía"],
      "Term of Service": ["Términos de servicio"],
      ok: ["ok"],
      changed: ["modificado"],
      "not accepted": ["no aceptado"],
      "unknown (exchange status should be updated)": [
        "desconocido (el estado del exchange debería actualizarse)",
      ],
      "Add an exchange": ["Agregar un exchange"],
      Troubleshooting: ["Solución de problemas"],
      "Developer mode": ["Modo desarrollador"],
      "More options and information useful for debugging": [
        "Más información y opciones útiles para depuración",
      ],
      Display: ["Pantalla"],
      "Current Language": ["Lenguaje actual"],
      "Wallet Core": ["Wallet core"],
      "Web Extension": ["Web Extension"],
      "Exchange compatibility": ["Compatibilidad con Exchange"],
      "Merchant compatibility": ["Compatibilidad con Merchant"],
      "Bank compatibility": ["Compatibilidad con Bank"],
      "Browser Extension Installed!": ["Extensión del navegador instalada!"],
      "You can open the GNU Taler Wallet using the combination %1$s .": [
        "Puedes abrir GNU Taler Wallet usando la combinación %1$s.",
      ],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [
          "También fijando GNU Taler Wallet a to navegador Chrome permite un acceso rápido sin el teclado:",
        ],
      "Click the puzzle icon": ["Haz click en el ícono de rompecabezas"],
      "Search for GNU Taler Wallet": ['Busca "GNU Taler Wallet"'],
      "Click the pin icon": ["Haz click en el ícono de fijar"],
      Permissions: ["Permisos"],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [
          "(Habilitar esta opción de abajo hará el uso de la billetera mas rápido, pero requiere mas permisos de tu navegador)",
        ],
      "Next Steps": ["Próximos pasos"],
      "Try the demo": ["Probar la demostración"],
      "Learn how to top up your wallet balance": [
        "Aprender como llenar tu billetera",
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "El diagnóstico caducó. No nos pudimos comunicar con la billetera.",
      ],
      "Problems detected:": ["Problemas detectados:"],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [
          "Por favor revisa en tu configuración %1$s que tienes IndexedDB habilitado (el nombre de la preferencia %2$s).",
        ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [
          "La base de datos de la billetera expiró. Por ahora la migración automática no está soportada. Por favor dirijasé a %1$s para reiniciar la base de datos de la billetera.",
        ],
      "Running diagnostics": ["Ejecutando diagnósticos"],
      "Debug tools": ["Herramientas de desarrollo"],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [
          "Quieres DESTRUIR IRREVOCABLEMENTE todo dentro de tu billetera y PERDER TODAS TUS MONEDAS?",
        ],
      reset: ["Reiniciar"],
      "TESTING: This may delete all your coin, proceed with caution": [
        "TESTING: Esto puede borrar todas tus monedas, proceder con precaución",
      ],
      "run gc": ["Ejecutar GC"],
      "import database": ["importar base de datos"],
      "export database": ["exportar base de datos"],
      "Database exported at %1$s %2$s to download": [
        "Base de datos exportada a %1$s %2$s para descargar",
      ],
      Coins: ["Monedas"],
      "Pending operations": ["Operaciones pendientes"],
      "usable coins": ["monedas usables"],
      id: ["id"],
      denom: ["denominación"],
      value: ["valor"],
      status: ["estado"],
      "from refresh?": ["desde refresco?"],
      "age key count": ["cantidad de age key"],
      "spent coins": ["monedas gastadas"],
      "click to show": ["hacer clic para mostrar"],
      "Scan a QR code or enter taler:// URI below": [
        "Escanear un código QR o ingresar taler:// URI debajo",
      ],
      Open: ["Abrir"],
      "URI is not valid. Taler URI should start with `taler://`": [
        "El URI no es válido. Taler URI debería comenzar con `taler://`",
      ],
      "Try another": ["Intentar otro"],
      "Could not load list of exchange": [
        "No se pudo cargar la lista de exchange",
      ],
      "Choose a currency to proceed or add another exchange": [
        "Elija una divisa para proceder o agregue otro exchange",
      ],
      "Known currencies": ["Divisas conocidas"],
      "Specify the amount and the origin": ["Indicar el monto y el origen"],
      "Change currency": ["Cambiar divisa"],
      "Use previous origins:": ["Usar un origen previo:"],
      "Or specify the origin of the money": [
        "O especificar el origen del dinero",
      ],
      "Specify the origin of the money": ["Especificar el origen del dinero"],
      "From my bank account": ["Desde mi cuenta de banco"],
      "From another wallet": ["Desde otra billetera"],
      "currency not provided": ["Divisa no provista"],
      "Specify the amount and the destination": [
        "Especificar el monto y el destino",
      ],
      "Use previous destinations:": ["Usar destinos previos:"],
      "Or specify the destination of the money": [
        "O especificar el destino del dinero",
      ],
      "Specify the destination of the money": [
        "Especificar el destino del dinero",
      ],
      "To my bank account": ["Hacia mi cuenta de banco"],
      "To another wallet": ["Hacia otra billetera"],
      "Could not load backup recovery information": [
        "No se pudo cargar la información de recuperación de copia de seguridad",
      ],
      "Digital wallet recovery": ["Recuperación de billetera digital"],
      "Import backup, show info": [
        "Importar copia de seguridad, mostrar información",
      ],
      "All done, your transaction is in progress": [
        "Todo completo, su transacción está en progreso",
      ],
      Edit: ["Editar"],
      "Could not load the list of known exchanges": [
        "No se pudo cargar la lista de exchange conocidos",
      ],
    },
  },
};

strings["fr"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=(n!=1);",
        lang: "fr",
      },
      Balance: [""],
      Backup: [""],
      "QR Reader and Taler URI": [""],
      Settings: [""],
      Dev: [""],
      "%1$s": [""],
      "PENDING OPERATIONS": [""],
      Loading: [""],
      "Could not load backup providers": [""],
      "No backup providers configured": [""],
      "Add provider": [""],
      "Sync all backups": [""],
      "Sync now": [""],
      "Last synced": [""],
      "Not synced": [""],
      "Expires in": [""],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        "",
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [""],
      "See providers": [""],
      "Last backup": [""],
      "Back up": [""],
      "Provider fee": [""],
      "per year": [""],
      Extend: [""],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [""],
      old: [""],
      new: [""],
      fee: [""],
      storage: [""],
      "Remove provider": [""],
      "This provider has reported an error": [""],
      "There is conflict with another backup from %1$s": [""],
      "Backup is not readable": [""],
      "Unknown backup problem: %1$s": [""],
      "service paid": [""],
      "Backup valid until": [""],
      Cancel: [""],
      "Open reserve page": [""],
      "Open pay page": [""],
      "Open refund page": [""],
      "Open tip page": [""],
      "Open withdraw page": [""],
      "Get digital cash": [""],
      "Could not load balance page": [""],
      Add: [""],
      "Send %1$s": [""],
      "Taler Action": [""],
      "This page has pay action.": [""],
      "This page has a withdrawal action.": [""],
      "This page has a tip action.": [""],
      "This page has a notify reserve action.": [""],
      Notify: [""],
      "This page has a refund action.": [""],
      "This page has a malformed taler uri.": [""],
      Dismiss: [""],
      "this popup is being closed and you are being redirected to %1$s": [""],
      "Could not load purchase proposal details": [""],
      "Order Id": [""],
      Summary: [""],
      Amount: [""],
      "Merchant name": [""],
      "Merchant jurisdiction": [""],
      "Merchant address": [""],
      "Merchant logo": [""],
      "Merchant website": [""],
      "Merchant email": [""],
      "Merchant public key": [""],
      "Delivery date": [""],
      "Delivery location": [""],
      Products: [""],
      "Created at": [""],
      "Refund deadline": [""],
      "Auto refund": [""],
      "Pay deadline": [""],
      "Fulfillment URL": [""],
      "Fulfillment message": [""],
      "Max deposit fee": [""],
      "Max fee": [""],
      "Minimum age": [""],
      "Wire fee amortization": [""],
      Auditors: [""],
      Exchanges: [""],
      "Bank account": [""],
      "Bitcoin address": [""],
      IBAN: [""],
      "Could not load deposit status": [""],
      "Digital cash deposit": [""],
      Cost: [""],
      Fee: [""],
      "To be received": [""],
      "Send &nbsp; %1$s": [""],
      "Bitcoin transfer details": [""],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [""],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [""],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [""],
      Account: [""],
      "Bank host": [""],
      "Bank transfer details": [""],
      Subject: [""],
      "Receiver name": [""],
      "Could not load the transaction information": [""],
      "There was an error trying to complete the transaction": [""],
      "This transaction is not completed": [""],
      Send: [""],
      Retry: [""],
      Forget: [""],
      "Caution!": [""],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [""],
      Confirm: ["Confirmer"],
      Withdrawal: [""],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [""],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [""],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [""],
      Details: [""],
      Payment: [""],
      Refunds: [""],
      "%1$s %2$s on %3$s": [""],
      "Merchant created a refund for this order but was not automatically picked up.":
        [""],
      Offer: [""],
      Accept: [""],
      Merchant: [""],
      "Invoice ID": [""],
      Deposit: [""],
      Refresh: [""],
      Tip: [""],
      Refund: [""],
      "Original order ID": [""],
      "Purchase summary": [""],
      copy: [""],
      "hide qr": [""],
      "show qr": [""],
      Credit: [""],
      Invoice: [""],
      Exchange: [""],
      URI: [""],
      Debit: [""],
      Transfer: [""],
      Country: [""],
      "Address lines": [""],
      "Building number": [""],
      "Building name": [""],
      Street: [""],
      "Post code": [""],
      "Town location": [""],
      Town: [""],
      District: [""],
      "Country subdivision": [""],
      Date: [""],
      "Transaction fees": [""],
      Total: [""],
      Withdraw: [""],
      Price: [""],
      Refunded: [""],
      Delivery: [""],
      "Total transfer": [""],
      "Could not load pay status": [""],
      "Digital cash payment": [""],
      Purchase: [""],
      Receipt: [""],
      "Valid until": [""],
      "List of products": [""],
      free: [""],
      "Already paid, you are going to be redirected to %1$s": [""],
      "Already paid": [""],
      "Already claimed": [""],
      "Pay with a mobile phone": [""],
      "Hide QR": [""],
      "Scan the QR code or &nbsp; %1$s": [""],
      "Pay &nbsp; %1$s": [""],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [""],
      "Your current balance is not enough.": [""],
      "Merchant message": [""],
      "Could not load refund status": [""],
      "Digital cash refund": [""],
      "You&apos;ve ignored the tip.": [""],
      "The refund is in progress.": [""],
      "Total to refund": [""],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [""],
      "Order amount": [""],
      "Already refunded": [""],
      "Refund offered": [""],
      "Accept &nbsp; %1$s": [""],
      "Could not load tip status": [""],
      "Digital cash tip": [""],
      "The merchant is offering you a tip": [""],
      "Merchant URL": [""],
      "Receive &nbsp; %1$s": [""],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [""],
      "Select one option": [""],
      "Could not load": [""],
      "Show terms of service": [""],
      "I accept the exchange terms of service": [""],
      "Exchange doesn&apos;t have terms of service": [""],
      "Review exchange terms of service": [""],
      "Review new version of terms of service": [""],
      "The exchange reply with a empty terms of service": [""],
      "Download Terms of Service": [""],
      "Hide terms of service": [""],
      "Could not load exchange fees": [""],
      Close: [""],
      "could not find any exchange": [""],
      "could not find any exchange for the currency %1$s": [""],
      "Service fee description": [""],
      "Select %1$s exchange": [""],
      Reset: [""],
      "Use this exchange": [""],
      "Doesn&apos;t have auditors": [""],
      currency: [""],
      Operations: [""],
      Deposits: [""],
      Denomination: [""],
      Until: [""],
      Withdrawals: [""],
      Currency: [""],
      "Coin operations": [""],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [""],
      "Transfer operations": [""],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [""],
      Operation: [""],
      "Wallet operations": [""],
      Feature: [""],
      "Could not get the info from the URI": [""],
      "Could not get info of withdrawal": [""],
      "Digital cash withdrawal": [""],
      "Could not finish the withdrawal operation": [""],
      "Age restriction": [""],
      "Withdraw &nbsp; %1$s": [""],
      "Withdraw to a mobile phone": [""],
      "Digital invoice": [""],
      "Could not finish the invoice creation": [""],
      Create: [""],
      "Could not finish the payment operation": [""],
      "Digital cash transfer": [""],
      "Could not finish the transfer creation": [""],
      "Could not finish the pickup operation": [""],
      "Manual Withdrawal for %1$s": [""],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [""],
      "No exchange found for %1$s": [""],
      "Add Exchange": [""],
      "No exchange configured": [""],
      "Can&apos;t create the reserve": [""],
      "Start withdrawal": [""],
      "Could not load deposit balance": [""],
      "A currency or an amount should be indicated": [""],
      "There is no enough balance to make a deposit for currency %1$s": [""],
      "Send %1$s to your account": [""],
      "There is no account to make a deposit for currency %1$s": [""],
      "Add account": [""],
      "Select account": [""],
      "Add another account": [""],
      "Deposit fee": [""],
      "Total deposit": [""],
      "Deposit&nbsp;%1$s %2$s": [""],
      "Add bank account for %1$s": [""],
      "Enter the URL of an exchange you trust.": [""],
      "Unable add this account": [""],
      "Select account type": [""],
      "Review terms of service": [""],
      "Exchange URL": [""],
      "Add exchange": [""],
      "Add new exchange": [""],
      "Add exchange for %1$s": [""],
      "An exchange has been found! Review the information and click next": [""],
      "This exchange doesn&apos;t match the expected currency %1$s": [""],
      "Unable to verify this exchange": [""],
      "Unable to add this exchange": [""],
      loading: [""],
      Version: [""],
      Next: [""],
      "Waiting for confirmation": [""],
      PENDING: [""],
      "Could not load the list of transactions": [""],
      "Your transaction history is empty for this currency.": [""],
      "Add backup provider": [""],
      "Could not get provider information": [""],
      "Backup providers may charge for their service": [""],
      URL: [""],
      Name: [""],
      "Provider URL": [""],
      "Please review and accept this provider&apos;s terms of service": [""],
      Pricing: [""],
      "free of charge": [""],
      "%1$s per year of service": [""],
      Storage: [""],
      "%1$s megabytes of storage per year of service": [""],
      "Accept terms of service": [""],
      "Could not parse the payto URI": [""],
      "Please check the uri": [""],
      "Exchange is ready for withdrawal": [""],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [""],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [""],
      "Cancel withdrawal": [""],
      "Could not toggle auto-open": [""],
      "Could not toggle clipboard": [""],
      Navigator: [""],
      "Automatically open wallet based on page content": [""],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [""],
      "Automatically check clipboard for Taler URI": [""],
      Trust: [""],
      "No exchange yet": [""],
      "Term of Service": [""],
      ok: [""],
      changed: [""],
      "not accepted": [""],
      "unknown (exchange status should be updated)": [""],
      "Add an exchange": [""],
      Troubleshooting: [""],
      "Developer mode": [""],
      "More options and information useful for debugging": [""],
      Display: [""],
      "Current Language": [""],
      "Wallet Core": [""],
      "Web Extension": [""],
      "Exchange compatibility": [""],
      "Merchant compatibility": [""],
      "Bank compatibility": [""],
      "Browser Extension Installed!": [""],
      "You can open the GNU Taler Wallet using the combination %1$s .": [""],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [""],
      "Click the puzzle icon": [""],
      "Search for GNU Taler Wallet": [""],
      "Click the pin icon": [""],
      Permissions: [""],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [""],
      "Next Steps": [""],
      "Try the demo": [""],
      "Learn how to top up your wallet balance": [""],
      "Diagnostics timed out. Could not talk to the wallet backend.": [""],
      "Problems detected:": [""],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [""],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [""],
      "Running diagnostics": [""],
      "Debug tools": [""],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [""],
      reset: [""],
      "TESTING: This may delete all your coin, proceed with caution": [""],
      "run gc": [""],
      "import database": [""],
      "export database": [""],
      "Database exported at %1$s %2$s to download": [""],
      Coins: [""],
      "Pending operations": [""],
      "usable coins": [""],
      id: [""],
      denom: [""],
      value: [""],
      status: [""],
      "from refresh?": [""],
      "age key count": [""],
      "spent coins": [""],
      "click to show": [""],
      "Scan a QR code or enter taler:// URI below": [""],
      Open: [""],
      "URI is not valid. Taler URI should start with `taler://`": [""],
      "Try another": [""],
      "Could not load list of exchange": [""],
      "Choose a currency to proceed or add another exchange": [""],
      "Known currencies": [""],
      "Specify the amount and the origin": [""],
      "Change currency": [""],
      "Use previous origins:": [""],
      "Or specify the origin of the money": [""],
      "Specify the origin of the money": [""],
      "From my bank account": [""],
      "From another wallet": [""],
      "currency not provided": [""],
      "Specify the amount and the destination": [""],
      "Use previous destinations:": [""],
      "Or specify the destination of the money": [""],
      "Specify the destination of the money": [""],
      "To my bank account": [""],
      "To another wallet": [""],
      "Could not load backup recovery information": [""],
      "Digital wallet recovery": [""],
      "Import backup, show info": [""],
      "All done, your transaction is in progress": [""],
      Edit: [""],
      "Could not load the list of known exchanges": [""],
    },
  },
};

strings["it"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=n != 1;",
        lang: "it",
      },
      Balance: [""],
      Backup: [""],
      "QR Reader and Taler URI": [""],
      Settings: [""],
      Dev: [""],
      "%1$s": [""],
      "PENDING OPERATIONS": [""],
      Loading: [""],
      "Could not load backup providers": [""],
      "No backup providers configured": [""],
      "Add provider": [""],
      "Sync all backups": [""],
      "Sync now": [""],
      "Last synced": [""],
      "Not synced": [""],
      "Expires in": [""],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        "",
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [""],
      "See providers": [""],
      "Last backup": [""],
      "Back up": [""],
      "Provider fee": [""],
      "per year": [""],
      Extend: [""],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [""],
      old: [""],
      new: [""],
      fee: [""],
      storage: [""],
      "Remove provider": [""],
      "This provider has reported an error": [""],
      "There is conflict with another backup from %1$s": [""],
      "Backup is not readable": [""],
      "Unknown backup problem: %1$s": [""],
      "service paid": [""],
      "Backup valid until": [""],
      Cancel: [""],
      "Open reserve page": [""],
      "Open pay page": [""],
      "Open refund page": [""],
      "Open tip page": [""],
      "Open withdraw page": [""],
      "Get digital cash": [""],
      "Could not load balance page": [""],
      Add: [""],
      "Send %1$s": [""],
      "Taler Action": [""],
      "This page has pay action.": [""],
      "This page has a withdrawal action.": [""],
      "This page has a tip action.": [""],
      "This page has a notify reserve action.": [""],
      Notify: [""],
      "This page has a refund action.": [""],
      "This page has a malformed taler uri.": [""],
      Dismiss: [""],
      "this popup is being closed and you are being redirected to %1$s": [""],
      "Could not load purchase proposal details": [""],
      "Order Id": [""],
      Summary: [""],
      Amount: [""],
      "Merchant name": [""],
      "Merchant jurisdiction": [""],
      "Merchant address": [""],
      "Merchant logo": [""],
      "Merchant website": [""],
      "Merchant email": [""],
      "Merchant public key": [""],
      "Delivery date": [""],
      "Delivery location": [""],
      Products: [""],
      "Created at": [""],
      "Refund deadline": [""],
      "Auto refund": [""],
      "Pay deadline": [""],
      "Fulfillment URL": [""],
      "Fulfillment message": [""],
      "Max deposit fee": [""],
      "Max fee": [""],
      "Minimum age": [""],
      "Wire fee amortization": [""],
      Auditors: [""],
      Exchanges: [""],
      "Bank account": [""],
      "Bitcoin address": [""],
      IBAN: [""],
      "Could not load deposit status": [""],
      "Digital cash deposit": [""],
      Cost: [""],
      Fee: [""],
      "To be received": [""],
      "Send &nbsp; %1$s": [""],
      "Bitcoin transfer details": [""],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [""],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [""],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [""],
      Account: [""],
      "Bank host": [""],
      "Bank transfer details": [""],
      Subject: [""],
      "Receiver name": [""],
      "Could not load the transaction information": [""],
      "There was an error trying to complete the transaction": [""],
      "This transaction is not completed": [""],
      Send: [""],
      Retry: [""],
      Forget: [""],
      "Caution!": [""],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [""],
      Confirm: ["Confermare"],
      Withdrawal: [""],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [""],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [""],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [""],
      Details: [""],
      Payment: [""],
      Refunds: [""],
      "%1$s %2$s on %3$s": [""],
      "Merchant created a refund for this order but was not automatically picked up.":
        [""],
      Offer: [""],
      Accept: [""],
      Merchant: [""],
      "Invoice ID": [""],
      Deposit: [""],
      Refresh: [""],
      Tip: [""],
      Refund: [""],
      "Original order ID": [""],
      "Purchase summary": [""],
      copy: [""],
      "hide qr": [""],
      "show qr": [""],
      Credit: [""],
      Invoice: [""],
      Exchange: [""],
      URI: [""],
      Debit: [""],
      Transfer: [""],
      Country: [""],
      "Address lines": [""],
      "Building number": [""],
      "Building name": [""],
      Street: [""],
      "Post code": [""],
      "Town location": [""],
      Town: [""],
      District: [""],
      "Country subdivision": [""],
      Date: [""],
      "Transaction fees": [""],
      Total: [""],
      Withdraw: [""],
      Price: [""],
      Refunded: [""],
      Delivery: [""],
      "Total transfer": [""],
      "Could not load pay status": [""],
      "Digital cash payment": [""],
      Purchase: [""],
      Receipt: [""],
      "Valid until": [""],
      "List of products": [""],
      free: [""],
      "Already paid, you are going to be redirected to %1$s": [""],
      "Already paid": [""],
      "Already claimed": [""],
      "Pay with a mobile phone": [""],
      "Hide QR": [""],
      "Scan the QR code or &nbsp; %1$s": [""],
      "Pay &nbsp; %1$s": [""],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [""],
      "Your current balance is not enough.": [""],
      "Merchant message": [""],
      "Could not load refund status": [""],
      "Digital cash refund": [""],
      "You&apos;ve ignored the tip.": [""],
      "The refund is in progress.": [""],
      "Total to refund": [""],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [""],
      "Order amount": [""],
      "Already refunded": [""],
      "Refund offered": [""],
      "Accept &nbsp; %1$s": [""],
      "Could not load tip status": [""],
      "Digital cash tip": [""],
      "The merchant is offering you a tip": [""],
      "Merchant URL": [""],
      "Receive &nbsp; %1$s": [""],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [""],
      "Select one option": [""],
      "Could not load": [""],
      "Show terms of service": [""],
      "I accept the exchange terms of service": [""],
      "Exchange doesn&apos;t have terms of service": [""],
      "Review exchange terms of service": [""],
      "Review new version of terms of service": [""],
      "The exchange reply with a empty terms of service": [""],
      "Download Terms of Service": [""],
      "Hide terms of service": [""],
      "Could not load exchange fees": [""],
      Close: [""],
      "could not find any exchange": [""],
      "could not find any exchange for the currency %1$s": [""],
      "Service fee description": [""],
      "Select %1$s exchange": [""],
      Reset: [""],
      "Use this exchange": [""],
      "Doesn&apos;t have auditors": [""],
      currency: [""],
      Operations: [""],
      Deposits: [""],
      Denomination: [""],
      Until: [""],
      Withdrawals: [""],
      Currency: [""],
      "Coin operations": [""],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [""],
      "Transfer operations": [""],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [""],
      Operation: [""],
      "Wallet operations": [""],
      Feature: [""],
      "Could not get the info from the URI": [""],
      "Could not get info of withdrawal": [""],
      "Digital cash withdrawal": [""],
      "Could not finish the withdrawal operation": [""],
      "Age restriction": [""],
      "Withdraw &nbsp; %1$s": [""],
      "Withdraw to a mobile phone": [""],
      "Digital invoice": [""],
      "Could not finish the invoice creation": [""],
      Create: [""],
      "Could not finish the payment operation": [""],
      "Digital cash transfer": [""],
      "Could not finish the transfer creation": [""],
      "Could not finish the pickup operation": [""],
      "Manual Withdrawal for %1$s": [""],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [""],
      "No exchange found for %1$s": [""],
      "Add Exchange": [""],
      "No exchange configured": [""],
      "Can&apos;t create the reserve": [""],
      "Start withdrawal": [""],
      "Could not load deposit balance": [""],
      "A currency or an amount should be indicated": [""],
      "There is no enough balance to make a deposit for currency %1$s": [""],
      "Send %1$s to your account": [""],
      "There is no account to make a deposit for currency %1$s": [""],
      "Add account": [""],
      "Select account": [""],
      "Add another account": [""],
      "Deposit fee": [""],
      "Total deposit": [""],
      "Deposit&nbsp;%1$s %2$s": [""],
      "Add bank account for %1$s": [""],
      "Enter the URL of an exchange you trust.": [""],
      "Unable add this account": [""],
      "Select account type": [""],
      "Review terms of service": [""],
      "Exchange URL": [""],
      "Add exchange": [""],
      "Add new exchange": [""],
      "Add exchange for %1$s": [""],
      "An exchange has been found! Review the information and click next": [""],
      "This exchange doesn&apos;t match the expected currency %1$s": [""],
      "Unable to verify this exchange": [""],
      "Unable to add this exchange": [""],
      loading: [""],
      Version: [""],
      Next: [""],
      "Waiting for confirmation": [""],
      PENDING: [""],
      "Could not load the list of transactions": [""],
      "Your transaction history is empty for this currency.": [""],
      "Add backup provider": [""],
      "Could not get provider information": [""],
      "Backup providers may charge for their service": [""],
      URL: [""],
      Name: [""],
      "Provider URL": [""],
      "Please review and accept this provider&apos;s terms of service": [""],
      Pricing: [""],
      "free of charge": [""],
      "%1$s per year of service": [""],
      Storage: [""],
      "%1$s megabytes of storage per year of service": [""],
      "Accept terms of service": [""],
      "Could not parse the payto URI": [""],
      "Please check the uri": [""],
      "Exchange is ready for withdrawal": [""],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [""],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [""],
      "Cancel withdrawal": [""],
      "Could not toggle auto-open": [""],
      "Could not toggle clipboard": [""],
      Navigator: [""],
      "Automatically open wallet based on page content": [""],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [""],
      "Automatically check clipboard for Taler URI": [""],
      Trust: [""],
      "No exchange yet": [""],
      "Term of Service": [""],
      ok: [""],
      changed: [""],
      "not accepted": [""],
      "unknown (exchange status should be updated)": [""],
      "Add an exchange": [""],
      Troubleshooting: [""],
      "Developer mode": [""],
      "More options and information useful for debugging": [""],
      Display: [""],
      "Current Language": [""],
      "Wallet Core": [""],
      "Web Extension": [""],
      "Exchange compatibility": [""],
      "Merchant compatibility": [""],
      "Bank compatibility": [""],
      "Browser Extension Installed!": [""],
      "You can open the GNU Taler Wallet using the combination %1$s .": [""],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [""],
      "Click the puzzle icon": [""],
      "Search for GNU Taler Wallet": [""],
      "Click the pin icon": [""],
      Permissions: [""],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [""],
      "Next Steps": [""],
      "Try the demo": [""],
      "Learn how to top up your wallet balance": [""],
      "Diagnostics timed out. Could not talk to the wallet backend.": [""],
      "Problems detected:": [""],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [""],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [""],
      "Running diagnostics": [""],
      "Debug tools": [""],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [""],
      reset: [""],
      "TESTING: This may delete all your coin, proceed with caution": [""],
      "run gc": [""],
      "import database": [""],
      "export database": [""],
      "Database exported at %1$s %2$s to download": [""],
      Coins: [""],
      "Pending operations": [""],
      "usable coins": [""],
      id: [""],
      denom: [""],
      value: [""],
      status: [""],
      "from refresh?": [""],
      "age key count": [""],
      "spent coins": [""],
      "click to show": [""],
      "Scan a QR code or enter taler:// URI below": [""],
      Open: [""],
      "URI is not valid. Taler URI should start with `taler://`": [""],
      "Try another": [""],
      "Could not load list of exchange": [""],
      "Choose a currency to proceed or add another exchange": [""],
      "Known currencies": [""],
      "Specify the amount and the origin": [""],
      "Change currency": [""],
      "Use previous origins:": [""],
      "Or specify the origin of the money": [""],
      "Specify the origin of the money": [""],
      "From my bank account": [""],
      "From another wallet": [""],
      "currency not provided": [""],
      "Specify the amount and the destination": [""],
      "Use previous destinations:": [""],
      "Or specify the destination of the money": [""],
      "Specify the destination of the money": [""],
      "To my bank account": [""],
      "To another wallet": [""],
      "Could not load backup recovery information": [""],
      "Digital wallet recovery": [""],
      "Import backup, show info": [""],
      "All done, your transaction is in progress": [""],
      Edit: [""],
      "Could not load the list of known exchanges": [""],
    },
  },
};

strings["ja"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=n != 1;",
        lang: "ja",
      },
      Balance: ["残高"],
      Backup: ["バックアップ"],
      "QR Reader and Taler URI": [""],
      Settings: ["設定"],
      Dev: [""],
      "%1$s": [""],
      "PENDING OPERATIONS": [""],
      Loading: [""],
      "Could not load backup providers": [""],
      "No backup providers configured": [""],
      "Add provider": [""],
      "Sync all backups": [""],
      "Sync now": [""],
      "Last synced": [""],
      "Not synced": [""],
      "Expires in": [""],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        "",
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [""],
      "See providers": [""],
      "Last backup": [""],
      "Back up": [""],
      "Provider fee": [""],
      "per year": [""],
      Extend: [""],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [""],
      old: [""],
      new: [""],
      fee: [""],
      storage: [""],
      "Remove provider": [""],
      "This provider has reported an error": [""],
      "There is conflict with another backup from %1$s": [""],
      "Backup is not readable": [""],
      "Unknown backup problem: %1$s": [""],
      "service paid": [""],
      "Backup valid until": [""],
      Cancel: [""],
      "Open reserve page": [""],
      "Open pay page": [""],
      "Open refund page": [""],
      "Open tip page": [""],
      "Open withdraw page": [""],
      "Get digital cash": [""],
      "Could not load balance page": [""],
      Add: [""],
      "Send %1$s": [""],
      "Taler Action": [""],
      "This page has pay action.": [""],
      "This page has a withdrawal action.": [""],
      "This page has a tip action.": [""],
      "This page has a notify reserve action.": [""],
      Notify: [""],
      "This page has a refund action.": [""],
      "This page has a malformed taler uri.": [""],
      Dismiss: [""],
      "this popup is being closed and you are being redirected to %1$s": [""],
      "Could not load purchase proposal details": [""],
      "Order Id": [""],
      Summary: [""],
      Amount: [""],
      "Merchant name": [""],
      "Merchant jurisdiction": [""],
      "Merchant address": [""],
      "Merchant logo": [""],
      "Merchant website": [""],
      "Merchant email": [""],
      "Merchant public key": [""],
      "Delivery date": [""],
      "Delivery location": [""],
      Products: [""],
      "Created at": [""],
      "Refund deadline": [""],
      "Auto refund": [""],
      "Pay deadline": [""],
      "Fulfillment URL": [""],
      "Fulfillment message": [""],
      "Max deposit fee": [""],
      "Max fee": [""],
      "Minimum age": [""],
      "Wire fee amortization": [""],
      Auditors: [""],
      Exchanges: [""],
      "Bank account": [""],
      "Bitcoin address": [""],
      IBAN: [""],
      "Could not load deposit status": [""],
      "Digital cash deposit": [""],
      Cost: [""],
      Fee: [""],
      "To be received": [""],
      "Send &nbsp; %1$s": [""],
      "Bitcoin transfer details": [""],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [""],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [""],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [""],
      Account: [""],
      "Bank host": [""],
      "Bank transfer details": [""],
      Subject: [""],
      "Receiver name": [""],
      "Could not load the transaction information": [""],
      "There was an error trying to complete the transaction": [""],
      "This transaction is not completed": [""],
      Send: [""],
      Retry: [""],
      Forget: [""],
      "Caution!": [""],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [""],
      Confirm: [""],
      Withdrawal: [""],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [""],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [""],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [""],
      Details: [""],
      Payment: [""],
      Refunds: [""],
      "%1$s %2$s on %3$s": [""],
      "Merchant created a refund for this order but was not automatically picked up.":
        [""],
      Offer: [""],
      Accept: [""],
      Merchant: [""],
      "Invoice ID": [""],
      Deposit: [""],
      Refresh: [""],
      Tip: [""],
      Refund: [""],
      "Original order ID": [""],
      "Purchase summary": [""],
      copy: [""],
      "hide qr": [""],
      "show qr": [""],
      Credit: [""],
      Invoice: [""],
      Exchange: [""],
      URI: [""],
      Debit: [""],
      Transfer: [""],
      Country: [""],
      "Address lines": [""],
      "Building number": [""],
      "Building name": [""],
      Street: [""],
      "Post code": [""],
      "Town location": [""],
      Town: [""],
      District: [""],
      "Country subdivision": [""],
      Date: [""],
      "Transaction fees": [""],
      Total: [""],
      Withdraw: ["撤退"],
      Price: [""],
      Refunded: [""],
      Delivery: [""],
      "Total transfer": [""],
      "Could not load pay status": [""],
      "Digital cash payment": [""],
      Purchase: [""],
      Receipt: [""],
      "Valid until": [""],
      "List of products": [""],
      free: [""],
      "Already paid, you are going to be redirected to %1$s": [""],
      "Already paid": [""],
      "Already claimed": [""],
      "Pay with a mobile phone": [""],
      "Hide QR": [""],
      "Scan the QR code or &nbsp; %1$s": [""],
      "Pay &nbsp; %1$s": [""],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [""],
      "Your current balance is not enough.": ["表示するバランスがありません"],
      "Merchant message": [""],
      "Could not load refund status": [""],
      "Digital cash refund": [""],
      "You&apos;ve ignored the tip.": [""],
      "The refund is in progress.": [""],
      "Total to refund": [""],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [""],
      "Order amount": [""],
      "Already refunded": [""],
      "Refund offered": [""],
      "Accept &nbsp; %1$s": [""],
      "Could not load tip status": [""],
      "Digital cash tip": [""],
      "The merchant is offering you a tip": [""],
      "Merchant URL": [""],
      "Receive &nbsp; %1$s": [""],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [""],
      "Select one option": [""],
      "Could not load": [""],
      "Show terms of service": [""],
      "I accept the exchange terms of service": [""],
      "Exchange doesn&apos;t have terms of service": [""],
      "Review exchange terms of service": [""],
      "Review new version of terms of service": [""],
      "The exchange reply with a empty terms of service": [""],
      "Download Terms of Service": [""],
      "Hide terms of service": [""],
      "Could not load exchange fees": [""],
      Close: [""],
      "could not find any exchange": [""],
      "could not find any exchange for the currency %1$s": [""],
      "Service fee description": [""],
      "Select %1$s exchange": [""],
      Reset: [""],
      "Use this exchange": [""],
      "Doesn&apos;t have auditors": [""],
      currency: [""],
      Operations: [""],
      Deposits: [""],
      Denomination: [""],
      Until: [""],
      Withdrawals: ["撤退"],
      Currency: [""],
      "Coin operations": [""],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [""],
      "Transfer operations": [""],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [""],
      Operation: [""],
      "Wallet operations": [""],
      Feature: [""],
      "Could not get the info from the URI": [""],
      "Could not get info of withdrawal": [""],
      "Digital cash withdrawal": [""],
      "Could not finish the withdrawal operation": [""],
      "Age restriction": [""],
      "Withdraw &nbsp; %1$s": [""],
      "Withdraw to a mobile phone": [""],
      "Digital invoice": [""],
      "Could not finish the invoice creation": [""],
      Create: [""],
      "Could not finish the payment operation": [""],
      "Digital cash transfer": [""],
      "Could not finish the transfer creation": [""],
      "Could not finish the pickup operation": [""],
      "Manual Withdrawal for %1$s": [""],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [""],
      "No exchange found for %1$s": [""],
      "Add Exchange": [""],
      "No exchange configured": [""],
      "Can&apos;t create the reserve": [""],
      "Start withdrawal": [""],
      "Could not load deposit balance": [""],
      "A currency or an amount should be indicated": [""],
      "There is no enough balance to make a deposit for currency %1$s": [""],
      "Send %1$s to your account": [""],
      "There is no account to make a deposit for currency %1$s": [""],
      "Add account": [""],
      "Select account": [""],
      "Add another account": [""],
      "Deposit fee": [""],
      "Total deposit": [""],
      "Deposit&nbsp;%1$s %2$s": [""],
      "Add bank account for %1$s": [""],
      "Enter the URL of an exchange you trust.": [""],
      "Unable add this account": [""],
      "Select account type": [""],
      "Review terms of service": [""],
      "Exchange URL": [""],
      "Add exchange": [""],
      "Add new exchange": [""],
      "Add exchange for %1$s": [""],
      "An exchange has been found! Review the information and click next": [""],
      "This exchange doesn&apos;t match the expected currency %1$s": [""],
      "Unable to verify this exchange": [""],
      "Unable to add this exchange": [""],
      loading: [""],
      Version: [""],
      Next: [""],
      "Waiting for confirmation": [""],
      PENDING: [""],
      "Could not load the list of transactions": [""],
      "Your transaction history is empty for this currency.": [""],
      "Add backup provider": [""],
      "Could not get provider information": [""],
      "Backup providers may charge for their service": [""],
      URL: [""],
      Name: [""],
      "Provider URL": [""],
      "Please review and accept this provider&apos;s terms of service": [""],
      Pricing: [""],
      "free of charge": [""],
      "%1$s per year of service": [""],
      Storage: [""],
      "%1$s megabytes of storage per year of service": [""],
      "Accept terms of service": [""],
      "Could not parse the payto URI": [""],
      "Please check the uri": [""],
      "Exchange is ready for withdrawal": [""],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [""],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [""],
      "Cancel withdrawal": [""],
      "Could not toggle auto-open": [""],
      "Could not toggle clipboard": [""],
      Navigator: [""],
      "Automatically open wallet based on page content": [""],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [""],
      "Automatically check clipboard for Taler URI": [""],
      Trust: [""],
      "No exchange yet": [""],
      "Term of Service": [""],
      ok: [""],
      changed: [""],
      "not accepted": [""],
      "unknown (exchange status should be updated)": [""],
      "Add an exchange": [""],
      Troubleshooting: [""],
      "Developer mode": [""],
      "More options and information useful for debugging": [""],
      Display: [""],
      "Current Language": [""],
      "Wallet Core": [""],
      "Web Extension": [""],
      "Exchange compatibility": [""],
      "Merchant compatibility": [""],
      "Bank compatibility": [""],
      "Browser Extension Installed!": [""],
      "You can open the GNU Taler Wallet using the combination %1$s .": [""],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [""],
      "Click the puzzle icon": [""],
      "Search for GNU Taler Wallet": [""],
      "Click the pin icon": [""],
      Permissions: [""],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [""],
      "Next Steps": [""],
      "Try the demo": [""],
      "Learn how to top up your wallet balance": [""],
      "Diagnostics timed out. Could not talk to the wallet backend.": [""],
      "Problems detected:": [""],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [""],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [""],
      "Running diagnostics": [""],
      "Debug tools": [""],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [""],
      reset: [""],
      "TESTING: This may delete all your coin, proceed with caution": [""],
      "run gc": [""],
      "import database": [""],
      "export database": [""],
      "Database exported at %1$s %2$s to download": [""],
      Coins: [""],
      "Pending operations": [""],
      "usable coins": [""],
      id: [""],
      denom: [""],
      value: [""],
      status: [""],
      "from refresh?": [""],
      "age key count": [""],
      "spent coins": [""],
      "click to show": [""],
      "Scan a QR code or enter taler:// URI below": [""],
      Open: [""],
      "URI is not valid. Taler URI should start with `taler://`": [""],
      "Try another": [""],
      "Could not load list of exchange": [""],
      "Choose a currency to proceed or add another exchange": [""],
      "Known currencies": [""],
      "Specify the amount and the origin": [""],
      "Change currency": [""],
      "Use previous origins:": [""],
      "Or specify the origin of the money": [""],
      "Specify the origin of the money": [""],
      "From my bank account": [""],
      "From another wallet": [""],
      "currency not provided": [""],
      "Specify the amount and the destination": [""],
      "Use previous destinations:": [""],
      "Or specify the destination of the money": [""],
      "Specify the destination of the money": [""],
      "To my bank account": [""],
      "To another wallet": [""],
      "Could not load backup recovery information": [""],
      "Digital wallet recovery": [""],
      "Import backup, show info": [""],
      "All done, your transaction is in progress": [""],
      Edit: [""],
      "Could not load the list of known exchanges": [""],
    },
  },
};

strings["sv"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=n != 1;",
        lang: "sv",
      },
      Balance: ["Balans"],
      Backup: [""],
      "QR Reader and Taler URI": [""],
      Settings: [""],
      Dev: [""],
      "%1$s": [""],
      "PENDING OPERATIONS": [""],
      Loading: [""],
      "Could not load backup providers": [""],
      "No backup providers configured": [""],
      "Add provider": [""],
      "Sync all backups": [""],
      "Sync now": [""],
      "Last synced": [""],
      "Not synced": [""],
      "Expires in": [""],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        "",
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [""],
      "See providers": [""],
      "Last backup": [""],
      "Back up": [""],
      "Provider fee": [""],
      "per year": [""],
      Extend: [""],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [""],
      old: [""],
      new: [""],
      fee: [""],
      storage: [""],
      "Remove provider": [""],
      "This provider has reported an error": [""],
      "There is conflict with another backup from %1$s": [""],
      "Backup is not readable": [""],
      "Unknown backup problem: %1$s": [""],
      "service paid": [""],
      "Backup valid until": [""],
      Cancel: ["Avbryt"],
      "Open reserve page": [""],
      "Open pay page": [""],
      "Open refund page": [""],
      "Open tip page": [""],
      "Open withdraw page": ["Utbetalnings avgift"],
      "Get digital cash": ["Utbetalnings avgifter:"],
      "Could not load balance page": [""],
      Add: [""],
      "Send %1$s": ["Välj %1$s"],
      "Taler Action": [""],
      "This page has pay action.": [""],
      "This page has a withdrawal action.": [""],
      "This page has a tip action.": [""],
      "This page has a notify reserve action.": [""],
      Notify: [""],
      "This page has a refund action.": [""],
      "This page has a malformed taler uri.": [""],
      Dismiss: [""],
      "this popup is being closed and you are being redirected to %1$s": [""],
      "Could not load purchase proposal details": [""],
      "Order Id": [""],
      Summary: [""],
      Amount: [""],
      "Merchant name": [""],
      "Merchant jurisdiction": [""],
      "Merchant address": [""],
      "Merchant logo": [""],
      "Merchant website": [""],
      "Merchant email": [""],
      "Merchant public key": [""],
      "Delivery date": [""],
      "Delivery location": [""],
      Products: [""],
      "Created at": [""],
      "Refund deadline": [""],
      "Auto refund": [""],
      "Pay deadline": [""],
      "Fulfillment URL": [""],
      "Fulfillment message": [""],
      "Max deposit fee": ["Depostitions avgift"],
      "Max fee": [""],
      "Minimum age": [""],
      "Wire fee amortization": [""],
      Auditors: [""],
      Exchanges: ["Accepterade tjänsteleverantörer:"],
      "Bank account": ["Övervisa till bank konto"],
      "Bitcoin address": [""],
      IBAN: [""],
      "Could not load deposit status": [""],
      "Digital cash deposit": [""],
      Cost: [""],
      Fee: [""],
      "To be received": [""],
      "Send &nbsp; %1$s": [""],
      "Bitcoin transfer details": [""],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [""],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [""],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [""],
      Account: [""],
      "Bank host": [""],
      "Bank transfer details": [""],
      Subject: [""],
      "Receiver name": [""],
      "Could not load the transaction information": [""],
      "There was an error trying to complete the transaction": [""],
      "This transaction is not completed": [""],
      Send: [""],
      Retry: [""],
      Forget: [""],
      "Caution!": [""],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [""],
      Confirm: ["Bekräfta"],
      Withdrawal: ["Utbetalnings avgift"],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [""],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [""],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [""],
      Details: [""],
      Payment: ["Godkän betalning"],
      Refunds: [""],
      "%1$s %2$s on %3$s": ["Säljaren %1$sgav en återbetalning på %2$s.\n"],
      "Merchant created a refund for this order but was not automatically picked up.":
        [""],
      Offer: [""],
      Accept: [""],
      Merchant: [""],
      "Invoice ID": [""],
      Deposit: ["Depostitions avgift"],
      Refresh: ["Återhämtnings avgift"],
      Tip: [""],
      Refund: [""],
      "Original order ID": [""],
      "Purchase summary": [""],
      copy: [""],
      "hide qr": [""],
      "show qr": [""],
      Credit: [""],
      Invoice: [""],
      Exchange: [""],
      URI: [""],
      Debit: [""],
      Transfer: [""],
      Country: [""],
      "Address lines": [""],
      "Building number": [""],
      "Building name": [""],
      Street: [""],
      "Post code": [""],
      "Town location": [""],
      Town: [""],
      District: [""],
      "Country subdivision": [""],
      Date: [""],
      "Transaction fees": [""],
      Total: [""],
      Withdraw: ["Utbetalnings avgift"],
      Price: [""],
      Refunded: [""],
      Delivery: [""],
      "Total transfer": ["Utbetalnings avgift"],
      "Could not load pay status": [""],
      "Digital cash payment": [""],
      Purchase: [""],
      Receipt: [""],
      "Valid until": [""],
      "List of products": [""],
      free: [""],
      "Already paid, you are going to be redirected to %1$s": [""],
      "Already paid": [""],
      "Already claimed": [""],
      "Pay with a mobile phone": [""],
      "Hide QR": [""],
      "Scan the QR code or &nbsp; %1$s": [""],
      "Pay &nbsp; %1$s": [""],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [""],
      "Your current balance is not enough.": [
        "Du har ingen balans att visa. Behöver du\n %1$s att börja?\n",
      ],
      "Merchant message": [""],
      "Could not load refund status": [""],
      "Digital cash refund": [""],
      "You&apos;ve ignored the tip.": [""],
      "The refund is in progress.": [""],
      "Total to refund": ["Utbetalnings avgift"],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Säljaren %1$s erbjuder följande:",
      ],
      "Order amount": ["Återhämtnings avgift"],
      "Already refunded": [""],
      "Refund offered": [""],
      "Accept &nbsp; %1$s": [""],
      "Could not load tip status": [""],
      "Digital cash tip": [""],
      "The merchant is offering you a tip": [
        "Säljaren %1$s erbjuder följande:",
      ],
      "Merchant URL": [""],
      "Receive &nbsp; %1$s": [""],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [""],
      "Select one option": [""],
      "Could not load": [""],
      "Show terms of service": [""],
      "I accept the exchange terms of service": [""],
      "Exchange doesn&apos;t have terms of service": [""],
      "Review exchange terms of service": [""],
      "Review new version of terms of service": [""],
      "The exchange reply with a empty terms of service": [""],
      "Download Terms of Service": [""],
      "Hide terms of service": [""],
      "Could not load exchange fees": [""],
      Close: [""],
      "could not find any exchange": ["Accepterade tjänsteleverantörer:"],
      "could not find any exchange for the currency %1$s": [""],
      "Service fee description": [""],
      "Select %1$s exchange": ["Välj %1$s"],
      Reset: [""],
      "Use this exchange": ["Accepterade tjänsteleverantörer:"],
      "Doesn&apos;t have auditors": [""],
      currency: [""],
      Operations: [""],
      Deposits: ["Depostitions avgift"],
      Denomination: [""],
      Until: [""],
      Withdrawals: ["Utbetalnings avgift"],
      Currency: [""],
      "Coin operations": [""],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [""],
      "Transfer operations": [""],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [""],
      Operation: [""],
      "Wallet operations": [""],
      Feature: [""],
      "Could not get the info from the URI": [""],
      "Could not get info of withdrawal": [""],
      "Digital cash withdrawal": [""],
      "Could not finish the withdrawal operation": [""],
      "Age restriction": [""],
      "Withdraw &nbsp; %1$s": [""],
      "Withdraw to a mobile phone": [""],
      "Digital invoice": [""],
      "Could not finish the invoice creation": [""],
      Create: [""],
      "Could not finish the payment operation": [""],
      "Digital cash transfer": [""],
      "Could not finish the transfer creation": [""],
      "Could not finish the pickup operation": [""],
      "Manual Withdrawal for %1$s": ["Utbetalnings avgift"],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [""],
      "No exchange found for %1$s": ["Accepterade tjänsteleverantörer:"],
      "Add Exchange": ["Accepterade tjänsteleverantörer:"],
      "No exchange configured": [""],
      "Can&apos;t create the reserve": [""],
      "Start withdrawal": [""],
      "Could not load deposit balance": [""],
      "A currency or an amount should be indicated": [""],
      "There is no enough balance to make a deposit for currency %1$s": [""],
      "Send %1$s to your account": [""],
      "There is no account to make a deposit for currency %1$s": [""],
      "Add account": ["Övervisa till bank konto"],
      "Select account": ["Övervisa till bank konto"],
      "Add another account": ["Övervisa till bank konto"],
      "Deposit fee": ["Depostitions avgift"],
      "Total deposit": [""],
      "Deposit&nbsp;%1$s %2$s": ["Depostitions avgift"],
      "Add bank account for %1$s": ["Accepterade tjänsteleverantörer:"],
      "Enter the URL of an exchange you trust.": [""],
      "Unable add this account": [""],
      "Select account type": [""],
      "Review terms of service": [""],
      "Exchange URL": [""],
      "Add exchange": ["Accepterade tjänsteleverantörer:"],
      "Add new exchange": ["Accepterade tjänsteleverantörer:"],
      "Add exchange for %1$s": ["Accepterade tjänsteleverantörer:"],
      "An exchange has been found! Review the information and click next": [""],
      "This exchange doesn&apos;t match the expected currency %1$s": [""],
      "Unable to verify this exchange": [""],
      "Unable to add this exchange": [""],
      loading: [""],
      Version: [""],
      Next: [""],
      "Waiting for confirmation": [""],
      PENDING: [""],
      "Could not load the list of transactions": [""],
      "Your transaction history is empty for this currency.": [""],
      "Add backup provider": [""],
      "Could not get provider information": [""],
      "Backup providers may charge for their service": [""],
      URL: [""],
      Name: [""],
      "Provider URL": [""],
      "Please review and accept this provider&apos;s terms of service": [""],
      Pricing: [""],
      "free of charge": [""],
      "%1$s per year of service": [""],
      Storage: [""],
      "%1$s megabytes of storage per year of service": [""],
      "Accept terms of service": [""],
      "Could not parse the payto URI": [""],
      "Please check the uri": [""],
      "Exchange is ready for withdrawal": ["Tjänsteleverantörer i plånboken:"],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [""],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [""],
      "Cancel withdrawal": [""],
      "Could not toggle auto-open": [""],
      "Could not toggle clipboard": [""],
      Navigator: [""],
      "Automatically open wallet based on page content": [""],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [""],
      "Automatically check clipboard for Taler URI": [""],
      Trust: [""],
      "No exchange yet": [""],
      "Term of Service": [""],
      ok: [""],
      changed: [""],
      "not accepted": [""],
      "unknown (exchange status should be updated)": [""],
      "Add an exchange": ["Accepterade tjänsteleverantörer:"],
      Troubleshooting: [""],
      "Developer mode": [""],
      "More options and information useful for debugging": [""],
      Display: [""],
      "Current Language": [""],
      "Wallet Core": [""],
      "Web Extension": [""],
      "Exchange compatibility": [""],
      "Merchant compatibility": [""],
      "Bank compatibility": [""],
      "Browser Extension Installed!": [""],
      "You can open the GNU Taler Wallet using the combination %1$s .": [""],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [""],
      "Click the puzzle icon": [""],
      "Search for GNU Taler Wallet": [""],
      "Click the pin icon": [""],
      Permissions: [""],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [""],
      "Next Steps": [""],
      "Try the demo": [""],
      "Learn how to top up your wallet balance": [""],
      "Diagnostics timed out. Could not talk to the wallet backend.": [""],
      "Problems detected:": [""],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [""],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [""],
      "Running diagnostics": [""],
      "Debug tools": [""],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [""],
      reset: [""],
      "TESTING: This may delete all your coin, proceed with caution": [""],
      "run gc": [""],
      "import database": [""],
      "export database": [""],
      "Database exported at %1$s %2$s to download": [""],
      Coins: ["# Mynt"],
      "Pending operations": [""],
      "usable coins": [""],
      id: [""],
      denom: [""],
      value: ["Värde"],
      status: [""],
      "from refresh?": [""],
      "age key count": ["Övervisa till bank konto"],
      "spent coins": [""],
      "click to show": [""],
      "Scan a QR code or enter taler:// URI below": [""],
      Open: [""],
      "URI is not valid. Taler URI should start with `taler://`": [""],
      "Try another": [""],
      "Could not load list of exchange": [""],
      "Choose a currency to proceed or add another exchange": [""],
      "Known currencies": [""],
      "Specify the amount and the origin": [""],
      "Change currency": [""],
      "Use previous origins:": [""],
      "Or specify the origin of the money": [""],
      "Specify the origin of the money": [""],
      "From my bank account": ["Övervisa till bank konto"],
      "From another wallet": [""],
      "currency not provided": [""],
      "Specify the amount and the destination": [""],
      "Use previous destinations:": [""],
      "Or specify the destination of the money": [""],
      "Specify the destination of the money": [""],
      "To my bank account": ["Övervisa till bank konto"],
      "To another wallet": [""],
      "Could not load backup recovery information": [""],
      "Digital wallet recovery": [""],
      "Import backup, show info": [""],
      "All done, your transaction is in progress": [""],
      Edit: [""],
      "Could not load the list of known exchanges": [""],
    },
  },
};

strings["tr"] = {
  domain: "messages",
  locale_data: {
    messages: {
      "": {
        domain: "messages",
        plural_forms: "nplurals=2; plural=n != 1;",
        lang: "tr",
      },
      Balance: ["Bakiye"],
      Backup: ["Yedekle"],
      "QR Reader and Taler URI": [""],
      Settings: ["Ayarlar"],
      Dev: ["Gelişim"],
      "%1$s": ["%1$s"],
      "PENDING OPERATIONS": [""],
      Loading: ["Yükleniyor"],
      "Could not load backup providers": [
        "Yedekleme sağlayıcıları yüklenemedi",
      ],
      "No backup providers configured": [
        "Yapılandırılmış yedekleme sağlayıcısı yok",
      ],
      "Add provider": ["Sağlayıcı ekle"],
      "Sync all backups": ["Tüm yedeklemeleri senkronize et"],
      "Sync now": ["Şimdi senkronize et"],
      "Last synced": ["Son Senkronizasyon"],
      "Not synced": ["Senkronize Edilmedi"],
      "Expires in": ["İçinde sona eriyor"],
      "There was an error loading the provider detail for &quot; %1$s&quot;": [
        "",
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [""],
      "See providers": ["Sağlayıcı ekle"],
      "Last backup": [""],
      "Back up": [""],
      "Provider fee": [""],
      "per year": [""],
      Extend: [""],
      "terms has changed, extending the service will imply accepting the new terms of service":
        [""],
      old: [""],
      new: [""],
      fee: [""],
      storage: [""],
      "Remove provider": [""],
      "This provider has reported an error": [""],
      "There is conflict with another backup from %1$s": [""],
      "Backup is not readable": [""],
      "Unknown backup problem: %1$s": [""],
      "service paid": [""],
      "Backup valid until": [""],
      Cancel: ["Bakiye"],
      "Open reserve page": ["Rezerv sayfasını açın"],
      "Open pay page": ["Ödeme sayfasını açın"],
      "Open refund page": ["Geri ödeme sayfasını açın"],
      "Open tip page": ["İkramiye sayfasını açın"],
      "Open withdraw page": ["Para çekme sayfasını açın"],
      "Get digital cash": [""],
      "Could not load balance page": ["Bakiye sayfası yüklenemedi"],
      Add: [""],
      "Send %1$s": ["%1$s seçin"],
      "Taler Action": ["Taler Eylemi"],
      "This page has pay action.": ["Bu sayfada ödeme eylemi var."],
      "This page has a withdrawal action.": [
        "Bu sayfada para çekme eylemi var.",
      ],
      "This page has a tip action.": ["Bu sayfada bir ikramiye eylemi var."],
      "This page has a notify reserve action.": [
        "Bu sayfada bir rezervasyon bildir eylemi var.",
      ],
      Notify: ["Bildirin"],
      "This page has a refund action.": [
        "Bu sayfada bir geri ödeme eylemi var.",
      ],
      "This page has a malformed taler uri.": [
        "Bu sayfada hatalı biçimlendirilmiş taler uri var.",
      ],
      Dismiss: ["Reddet"],
      "this popup is being closed and you are being redirected to %1$s": [""],
      "Could not load purchase proposal details": [
        "Yedekleme sağlayıcıları yüklenemedi",
      ],
      "Order Id": ["Sipariş reddedildi"],
      Summary: [""],
      Amount: [""],
      "Merchant name": [""],
      "Merchant jurisdiction": [""],
      "Merchant address": [""],
      "Merchant logo": [""],
      "Merchant website": [""],
      "Merchant email": [""],
      "Merchant public key": [""],
      "Delivery date": [""],
      "Delivery location": ["Taler Eylemi"],
      Products: [""],
      "Created at": [""],
      "Refund deadline": [""],
      "Auto refund": ["Ödeme iadesi"],
      "Pay deadline": [""],
      "Fulfillment URL": [""],
      "Fulfillment message": [""],
      "Max deposit fee": [""],
      "Max fee": [""],
      "Minimum age": [""],
      "Wire fee amortization": [""],
      Auditors: [""],
      Exchanges: ["Exchange"],
      "Bank account": [""],
      "Bitcoin address": [""],
      IBAN: [""],
      "Could not load deposit status": ["Bakiye sayfası yüklenemedi"],
      "Digital cash deposit": [""],
      Cost: [""],
      Fee: ["Ücretler"],
      "To be received": [""],
      "Send &nbsp; %1$s": [""],
      "Bitcoin transfer details": [""],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.":
        [""],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts":
        [""],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC":
        [""],
      Account: [""],
      "Bank host": [""],
      "Bank transfer details": [""],
      Subject: [""],
      "Receiver name": [""],
      "Could not load the transaction information": [""],
      "There was an error trying to complete the transaction": [""],
      "This transaction is not completed": [""],
      Send: [""],
      Retry: ["Yeniden deneyin"],
      Forget: [""],
      "Caution!": [""],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.":
        [""],
      Confirm: ["Onaylamak"],
      Withdrawal: ["Çekildi"],
      "Make sure to use the correct subject, otherwise the money will not arrive in this wallet.":
        [""],
      "The bank did not yet confirmed the wire transfer. Go to the %1$s %2$s and check there is no pending step.":
        [""],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins":
        [""],
      Details: [""],
      Payment: ["Ödeme gönderildi"],
      Refunds: [""],
      "%1$s %2$s on %3$s": [""],
      "Merchant created a refund for this order but was not automatically picked up.":
        [""],
      Offer: [""],
      Accept: ["İkramiye kabul edildi"],
      Merchant: [""],
      "Invoice ID": [""],
      Deposit: [""],
      Refresh: ["Ücreti yenile"],
      Tip: [""],
      Refund: [""],
      "Original order ID": [""],
      "Purchase summary": [""],
      copy: [""],
      "hide qr": [""],
      "show qr": [""],
      Credit: [""],
      Invoice: [""],
      Exchange: ["Exchange"],
      URI: [""],
      Debit: [""],
      Transfer: [""],
      Country: [""],
      "Address lines": [""],
      "Building number": [""],
      "Building name": [""],
      Street: [""],
      "Post code": [""],
      "Town location": [""],
      Town: [""],
      District: [""],
      "Country subdivision": [""],
      Date: [""],
      "Transaction fees": [""],
      Total: [""],
      Withdraw: ["Para çek"],
      Price: [""],
      Refunded: [""],
      Delivery: [""],
      "Total transfer": ["Çekildi"],
      "Could not load pay status": [""],
      "Digital cash payment": [""],
      Purchase: [""],
      Receipt: [""],
      "Valid until": [""],
      "List of products": [""],
      free: [""],
      "Already paid, you are going to be redirected to %1$s": [""],
      "Already paid": [""],
      "Already claimed": [""],
      "Pay with a mobile phone": [""],
      "Hide QR": [""],
      "Scan the QR code or &nbsp; %1$s": [""],
      "Pay &nbsp; %1$s": [""],
      "You have no balance for this currency. Withdraw digital cash first.": [
        "",
      ],
      "Could not find enough coins to pay. Even if you have enough %1$s some restriction may apply.":
        [""],
      "Your current balance is not enough.": ["Gösterecek bakiyeniz yok."],
      "Merchant message": [""],
      "Could not load refund status": ["Bakiye sayfası yüklenemedi"],
      "Digital cash refund": [""],
      "You&apos;ve ignored the tip.": [""],
      "The refund is in progress.": [""],
      "Total to refund": ["Ödeme iadesi"],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [""],
      "Order amount": ["Ücreti yenile"],
      "Already refunded": ["Ödeme iadesi"],
      "Refund offered": [""],
      "Accept &nbsp; %1$s": [""],
      "Could not load tip status": ["Bakiye sayfası yüklenemedi"],
      "Digital cash tip": [""],
      "The merchant is offering you a tip": [""],
      "Merchant URL": [""],
      "Receive &nbsp; %1$s": [""],
      "Tip from %1$s accepted. Check your transactions list for more details.":
        [""],
      "Select one option": [""],
      "Could not load": ["Bakiye sayfası yüklenemedi"],
      "Show terms of service": ["Hizmet şartlarını göster"],
      "I accept the exchange terms of service": [
        "Hizmet şartlarını kabul ediyorum",
      ],
      "Exchange doesn&apos;t have terms of service": [
        "Exchange'in hizmet şartları yok",
      ],
      "Review exchange terms of service": [
        "Exchange'in hizmet şartlarını inceleyin",
      ],
      "Review new version of terms of service": [
        "Hizmet şartlarının yeni sürümünü inceleyin",
      ],
      "The exchange reply with a empty terms of service": [""],
      "Download Terms of Service": [""],
      "Hide terms of service": [""],
      "Could not load exchange fees": ["Bakiye sayfası yüklenemedi"],
      Close: [""],
      "could not find any exchange": ["Bakiye sayfası yüklenemedi"],
      "could not find any exchange for the currency %1$s": [""],
      "Service fee description": [""],
      "Select %1$s exchange": ["Özel exchange'i seçin"],
      Reset: [""],
      "Use this exchange": ["Özel exchange'i seçin"],
      "Doesn&apos;t have auditors": [""],
      currency: [""],
      Operations: ["Bekleyen işlemler"],
      Deposits: ["Depozito %1$s"],
      Denomination: ["Bekleyen işlemler"],
      Until: [""],
      Withdrawals: ["Çekildi"],
      Currency: [""],
      "Coin operations": ["Bekleyen işlemler"],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.":
        [""],
      "Transfer operations": ["Bekleyen işlemler"],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.":
        [""],
      Operation: [""],
      "Wallet operations": ["Bekleyen işlemler"],
      Feature: [""],
      "Could not get the info from the URI": [""],
      "Could not get info of withdrawal": [
        "Para çekme işlemi için ayrıntılar alınamadı:",
      ],
      "Digital cash withdrawal": [""],
      "Could not finish the withdrawal operation": [""],
      "Age restriction": [""],
      "Withdraw &nbsp; %1$s": [""],
      "Withdraw to a mobile phone": [""],
      "Digital invoice": [""],
      "Could not finish the invoice creation": [""],
      Create: [""],
      "Could not finish the payment operation": [
        "Para çekme işlemi için ayrıntılar alınamadı:",
      ],
      "Digital cash transfer": [""],
      "Could not finish the transfer creation": [
        "Para çekme işlemi için ayrıntılar alınamadı:",
      ],
      "Could not finish the pickup operation": [
        "Para çekme işlemi için ayrıntılar alınamadı:",
      ],
      "Manual Withdrawal for %1$s": ["Para çekme ücretleri:"],
      "Choose a exchange from where the coins will be withdrawn. The exchange will send the coins to this wallet after receiving a wire transfer with the correct subject.":
        [""],
      "No exchange found for %1$s": [""],
      "Add Exchange": [""],
      "No exchange configured": [""],
      "Can&apos;t create the reserve": [""],
      "Start withdrawal": [""],
      "Could not load deposit balance": ["Bakiye sayfası yüklenemedi"],
      "A currency or an amount should be indicated": [""],
      "There is no enough balance to make a deposit for currency %1$s": [""],
      "Send %1$s to your account": [""],
      "There is no account to make a deposit for currency %1$s": [""],
      "Add account": [""],
      "Select account": [""],
      "Add another account": [""],
      "Deposit fee": [""],
      "Total deposit": [""],
      "Deposit&nbsp;%1$s %2$s": ["Test Havale Hesap #%1$s üzerinde %2$s"],
      "Add bank account for %1$s": [""],
      "Enter the URL of an exchange you trust.": [""],
      "Unable add this account": [""],
      "Select account type": [""],
      "Review terms of service": [""],
      "Exchange URL": [""],
      "Add exchange": [""],
      "Add new exchange": [""],
      "Add exchange for %1$s": [""],
      "An exchange has been found! Review the information and click next": [""],
      "This exchange doesn&apos;t match the expected currency %1$s": [""],
      "Unable to verify this exchange": [""],
      "Unable to add this exchange": [""],
      loading: [""],
      Version: [""],
      Next: [""],
      "Waiting for confirmation": [""],
      PENDING: [""],
      "Could not load the list of transactions": [""],
      "Your transaction history is empty for this currency.": [""],
      "Add backup provider": [""],
      "Could not get provider information": [""],
      "Backup providers may charge for their service": [""],
      URL: [""],
      Name: [""],
      "Provider URL": [""],
      "Please review and accept this provider&apos;s terms of service": [
        "Hizmet şartlarını kabul ediyorum",
      ],
      Pricing: [""],
      "free of charge": [""],
      "%1$s per year of service": [""],
      Storage: [""],
      "%1$s megabytes of storage per year of service": [""],
      "Accept terms of service": [""],
      "Could not parse the payto URI": [""],
      "Please check the uri": [""],
      "Exchange is ready for withdrawal": [""],
      "To complete the process you need to wire%1$s %2$s to the exchange bank account":
        [""],
      "Alternative, you can also scan this QR code or open %1$s if you have a banking app installed that supports RFC 8905":
        [""],
      "Cancel withdrawal": [""],
      "Could not toggle auto-open": ["Bakiye sayfası yüklenemedi"],
      "Could not toggle clipboard": ["Bakiye sayfası yüklenemedi"],
      Navigator: [""],
      "Automatically open wallet based on page content": [""],
      "Enabling this option below will make using the wallet faster, but requires more permissions from your browser.":
        [""],
      "Automatically check clipboard for Taler URI": [""],
      Trust: [""],
      "No exchange yet": [""],
      "Term of Service": [""],
      ok: [""],
      changed: [""],
      "not accepted": [""],
      "unknown (exchange status should be updated)": [""],
      "Add an exchange": [""],
      Troubleshooting: [""],
      "Developer mode": [""],
      "More options and information useful for debugging": [""],
      Display: [""],
      "Current Language": [""],
      "Wallet Core": [""],
      "Web Extension": [""],
      "Exchange compatibility": [""],
      "Merchant compatibility": [""],
      "Bank compatibility": [""],
      "Browser Extension Installed!": [""],
      "You can open the GNU Taler Wallet using the combination %1$s .": [""],
      "Also pinning the GNU Taler Wallet to your Chrome browser allows you to quick access without keyboard:":
        [""],
      "Click the puzzle icon": [""],
      "Search for GNU Taler Wallet": [""],
      "Click the pin icon": [""],
      Permissions: [""],
      "(Enabling this option below will make using the wallet faster, but requires more permissions from your browser.)":
        [""],
      "Next Steps": [""],
      "Try the demo": [""],
      "Learn how to top up your wallet balance": [""],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "Tanılar zaman aşımına uğradı. Cüzdan arka ucuyla konuşulamadı.",
      ],
      "Problems detected:": ["Tespit edilen sorunlar:"],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).":
        [
          "Lütfen %1$s ayarlarınızda, IndexedDB'nin etkinleştirildiğinizi kontrol edin (%2$s tercih adını kontrol edin).",
        ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.":
        [
          "Cüzdan veritabanınız eski. Şu anda otomatik aktarım desteklenmiyor. Cüzdan veritabanını sıfırlamak için lütfen %1$s gidin.",
        ],
      "Running diagnostics": ["Tanılamayı çalıştır"],
      "Debug tools": ["Hata ayıklama araçları"],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?":
        [
          "Cüzdanınızdaki her şeyi GERİ ALINAMAZ BİÇİMDE İMHA ETMEK ve TÜM PARALARINIZI KAYBETMEK mi istiyorsunuz?",
        ],
      reset: ["sıfırla"],
      "TESTING: This may delete all your coin, proceed with caution": [""],
      "run gc": [""],
      "import database": ["veritabanını içe aktar"],
      "export database": ["veritabanını dışa aktar"],
      "Database exported at %1$s %2$s to download": [
        "Veritabanı %1$s'de dışa aktarıldı-%2$s indirilecek",
      ],
      Coins: ["Madeni paralar"],
      "Pending operations": ["Bekleyen işlemler"],
      "usable coins": ["kullanılabilir madeni paralar"],
      id: ["kimlik"],
      denom: [""],
      value: ["değer"],
      status: ["durum"],
      "from refresh?": ["yenilemeden mi?"],
      "age key count": [""],
      "spent coins": ["harcanan madeni paralar"],
      "click to show": ["göstermek için tıklayın"],
      "Scan a QR code or enter taler:// URI below": [""],
      Open: [""],
      "URI is not valid. Taler URI should start with `taler://`": [""],
      "Try another": [""],
      "Could not load list of exchange": ["Bakiye sayfası yüklenemedi"],
      "Choose a currency to proceed or add another exchange": [""],
      "Known currencies": [""],
      "Specify the amount and the origin": [""],
      "Change currency": [""],
      "Use previous origins:": [""],
      "Or specify the origin of the money": [""],
      "Specify the origin of the money": [""],
      "From my bank account": ["Banka hesabına havale yap"],
      "From another wallet": [""],
      "currency not provided": [""],
      "Specify the amount and the destination": [""],
      "Use previous destinations:": [""],
      "Or specify the destination of the money": [""],
      "Specify the destination of the money": [""],
      "To my bank account": ["Banka hesabına havale yap"],
      "To another wallet": [""],
      "Could not load backup recovery information": [
        "Yedekleme sağlayıcıları yüklenemedi",
      ],
      "Digital wallet recovery": [""],
      "Import backup, show info": [""],
      "All done, your transaction is in progress": [""],
      Edit: [
        "Lütfen bir exchange seçin. Detayları seçiminizden önce inceleyebilirsiniz.",
      ],
      "Could not load the list of known exchanges": [""],
    },
  },
};
