let x = <i18n.Translate>foo
{bar} {" foo "} baz</i18n.Translate>


let y = (
  <i18n.TranslateSwitch>
    <i18n.TranslateSingular>
      singular form
      second line
    </i18n.TranslateSingular>
    <i18n.TranslatePlural>
     plural form
    </i18n.TranslatePlural>
    </i18n.TranslateSwitch>
);

let z = <i18n.Translate>"foo"</i18n.Translate>;
