#!/bin/bash

[ ! -d prebuilt ] && git worktree add -f prebuilt prebuilt && exit 1

for file in offer_refund.en.html request_payment.en.html show_order_details.en.html; do
	cp packages/merchant-backend-ui/dist/pages/$file prebuilt/backend/
done

